﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Dayu
{
    class Editing
    {
        const bool PLEASE_CHECK = true;
        const int BLOCK_SIZE = 1024;
        const int DATA_ARRAY_LIMIT = 906;
        const int DOUBLE_KANJI = 4; // SJIS_KANJI_SIZE + SJIS_KANJI_SIZE
        const int ERROR_CODE = 1;
        const int FILE_ERROR_CODE = -2;
        const int FINAL_LINE = 16;
        const int FIRST_KANJI = 159;
        const int KANA_KUTEN = 161;
        const int KANA_HANDAKUTEN = 223;
        const int RECORD_COUNT = 7882;
        const int SEARCH_COUNT = 5;
        const int SEARCH_COUNT_1 = 4; // SEARCH_COUNT - 1
        const int SJIS_KANJI_SIZE = 2;
        const int YEN_SIZE = 1;
        const string C_MS932_SJIS_BIN = "C:\\sjisEUCjpCS\\MS932~Shift_JIS.bin";
        const string EMPTY = "";
        const string SETTING_TEXT = "twenty.txt";
        const string TEMPORARY_BINARY = "binary.tmp";
        const string RIGHT_ARROW = " ⇒ "; // UTF-8
        const string MS932_SJIS_BIN = "MS932~Shift_JIS.bin";
        const string YEN = "\\";

        delegate string OpenCloseFunction();
        NormalSubstitute[] data = new NormalSubstitute[DATA_ARRAY_LIMIT];

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        private int kanjiCode(string line)
        {
            bool kanji = false;
            Encoding MS932 = Encoding.GetEncoding(932);
            byte[] buffer = MS932.GetBytes(line);
            byte[] c = {0, 0, 0, 0};
            int before = 0;
            int k = 0;
            int n = 0;
            int size = MS932.GetByteCount(line);

            for (int i = 0; i < size; ++i)
            {
                k = (int)buffer[i];

                if (kanji)
                {
                    c[0] = buffer[before];
                    c[1] = buffer[i];
                    n = inverted(c);
                    break;
                }

                if (0x81 <= k && k <= 0xFC)
                {
                    if (KANA_KUTEN <= k && k <= KANA_HANDAKUTEN)
                    {
                        n = k;
                        break;
                    }
                    else
                    {
                        before = i;
                        kanji = true;
                    }

                }
                else
                {
                    n = k;
                    break;
                }

            }

            return n;
        }

        public bool editRecord(SearchTable s, string normal, string substitute, string binaryName, bool no_removing)
        {
            byte[] c = {0, 0, 0, 0}; // DOUBLE_KANJI
            byte[] combo = {0, 0, 0, 0}; // DOUBLE_KANJI
            int beginning = 0;
            int from, into, k;
            OpenCloseFunction[] openClose = new OpenCloseFunction[SEARCH_COUNT];
            string table = C_MS932_SJIS_BIN;

            SparseSJIS sjis = new SparseSJIS();
            if (! sjis.setting(s))
            {
                Console.Error.Write("editRecord() error: ");
                Console.Error.WriteLine("sjis.setting(s)");
                return false;
            }

            openClose[0] = binary_file_in_installed_directory;
            openClose[1] = binary_file_in_current_directory;
            openClose[2] = binary_file_in_path_directory;
            openClose[3] = locateBinaryAssembly;
            openClose[4] = binary_file_in_selected_directory;

            if ( s.priority() )
            {
                beginning = SEARCH_COUNT_1;
            }

            for (int i = beginning; i < SEARCH_COUNT; ++i)
            {
                table = openClose[i]();
                if ( table.Length > 0 )
                {
                    if ( s.whichTable() )
                    {
                        Console.WriteLine( table + " is primarily accessible to Diyo." );
                        s.whichTable(false);
                    }

                    break;
                }
                else
                {
                    switch(i)
                    {
                        case 0:
                            if( s.noSearching() ) {
                                Console.Error.Write("openClose[");
                                Console.Error.Write(i);
                                Console.Error.Write("]() error: ");
                                Console.Error.WriteLine(MS932_SJIS_BIN);
                                return false;
                            }

                            break;
                        case SEARCH_COUNT_1:
                            Console.Error.Write("openClose[");
                            Console.Error.Write(i);
                            Console.Error.Write("]() error: ");
                            Console.Error.WriteLine(MS932_SJIS_BIN);
                            return false;
                        default:
                            break;
                    }

                }

            }

            from = kanjiCode(normal);
            Console.Write( string.Format("{0:X4}", from) );
            Console.Write(RIGHT_ARROW);

            into = kanjiCode(substitute);
            Console.WriteLine( string.Format("{0:X4}", into) );

            try
            {
                File.Delete(TEMPORARY_BINARY);
            }
            catch (IOException e235)
            {
                Error.handler( e235.ToString() );
            }

            Stream binaryFile = File.OpenRead(table);
            Stream temporaryFile = File.OpenWrite(TEMPORARY_BINARY);
            for(int i = 1; i <= RECORD_COUNT; ++i)
            {
                binaryFile.Read(c, 0, DOUBLE_KANJI);
                if (i < FIRST_KANJI)
                {
                    k = (int)c[0];
                }
                else
                {
                    k = inverted(c);
                }

                combo[0] = c[0];
                combo[1] = c[1];

                if (k == from)
                {
                    combo[2] = sjis.working(into, 0);
                    combo[3] = sjis.working(into, 1);
                }
                else
                {
                    combo[2] = c[SJIS_KANJI_SIZE];
                    combo[3] = c[ SJIS_KANJI_SIZE + 1 ];
                }

                temporaryFile.Write(combo, 0, DOUBLE_KANJI);
            } // for(int i = 1; i <= RECORD_COUNT; ++i)

            binaryFile.Close();
            temporaryFile.Close();
            renameBinaryFile(binaryName, no_removing);
            return true;
        }

        public bool editManyRecords(SearchTable s, string text, string binaryName, bool no_removing)
        {
            bool b = true;
            bool found = false;
            byte[] c = {0, 0, 0, 0}; // DOUBLE_KANJI
            byte[] combo = {0, 0, 0, 0}; // DOUBLE_KANJI
            int beginning = 0;
            int from, k;
            int into = 0x5750;
            int used = 0;
            OpenCloseFunction[] openClose = new OpenCloseFunction[SEARCH_COUNT];
            string line, normal, substitute;
            string table = C_MS932_SJIS_BIN;

            try
            {
                StreamReader listFile = new StreamReader(text, Encoding.Default);
                for (int m = 0; m < DATA_ARRAY_LIMIT; ++m)
                {
                    line = listFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    normal = line.Substring(0, 1);
                    substitute = line.Substring(1, 1);
                    data[m] = new NormalSubstitute(normal, substitute, PLEASE_CHECK);
                    used = m;
                }

                listFile.Close();
            }
            catch (IOException e236)
            {
                Error.handler( e236.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(text);
                b = false;
            }
            catch (Exception e237)
            {
                Error.handler( e237.ToString() );
                b = false;
            }

            SparseSJIS sjis = new SparseSJIS();
            if (! sjis.setting(s))
            {
                Console.Error.Write("editManyRecords() error: ");
                Console.Error.WriteLine("sjis.setting(s)");
                return false;
            }

            openClose[0] = binary_file_in_installed_directory;
            openClose[1] = binary_file_in_current_directory;
            openClose[2] = binary_file_in_path_directory;
            openClose[3] = locateBinaryAssembly;
            openClose[4] = binary_file_in_selected_directory;

            if ( s.priority() )
            {
                beginning = SEARCH_COUNT_1;
            }

            for (int i = beginning; i < SEARCH_COUNT; ++i)
            {
                table = openClose[i]();
                if ( table.Length > 0 )
                {
                    if ( s.whichTable() )
                    {
                        Console.WriteLine( table + " is primarily accessible to Diyo." );
                        s.whichTable(false);
                    }

                    break;
                }
                else
                {
                    switch(i)
                    {
                        case 0:
                            if( s.noSearching() ) {
                                Console.Error.Write("openClose[");
                                Console.Error.Write(i);
                                Console.Error.Write("]() error: ");
                                Console.Error.WriteLine(MS932_SJIS_BIN);
                                return false;
                            }

                            break;
                        case SEARCH_COUNT_1:
                            Console.Error.Write("openClose[");
                            Console.Error.Write(i);
                            Console.Error.Write("]() error: ");
                            Console.Error.WriteLine(MS932_SJIS_BIN);
                            return false;
                        default:
                            break;
                    }

                }

            }

            try
            {
                File.Delete(TEMPORARY_BINARY);
                Stream binaryFile = File.OpenRead(table);
                Stream temporaryFile = File.OpenWrite(TEMPORARY_BINARY);

                for (int i = 1; i <= RECORD_COUNT; ++i)
                {
                    binaryFile.Read(c, 0, DOUBLE_KANJI);
                    if (i < FIRST_KANJI)
                    {
                        k = (int)c[0];
                    }
                    else
                    {
                        k = inverted(c);
                    }

                    combo[0] = c[0];
                    combo[1] = c[1];

                    for (int m = 0; m <= used; ++m)
                    {
                        if (data[m].pleaseCheck())
                        {
                            from = kanjiCode(data[m].normalCharacter());
                            if (k == from)
                            {
                                into = kanjiCode(data[m].substituteCharacter());
                                found = true;
                                data[m].sure();
                                break;
                            }
                            else
                            {
                                found = false;
                            }

                        }
                        else
                        {
                            found = false;
                        }

                    }

                    if (found)
                    {
                        combo[2] = sjis.working(into, 0);
                        combo[3] = sjis.working(into, 1);
                    }
                    else
                    {
                        combo[2] = c[SJIS_KANJI_SIZE];
                        combo[3] = c[SJIS_KANJI_SIZE + 1];
                    }

                    temporaryFile.Write(combo, 0, DOUBLE_KANJI);
                } // for (int i = 1; i <= RECORD_COUNT; ++i)

                binaryFile.Close();
                temporaryFile.Close();
                renameBinaryFile(binaryName, no_removing);
            }
            catch (IOException e238)
            {
                Error.handler( e238.ToString() );
                Console.Error.WriteLine(text);
                b = false;
            }
            catch (Exception e239)
            {
                Error.handler( e239.ToString() );
                b = false;
            }

            return b;
        }

        public int renameBinaryFile(string name, bool no_removing)
        {
            int exit_code = 0;
            int size;

            byte[] buffer = new byte[BLOCK_SIZE];

            try
            {
                File.Delete(name);
                Stream temporary = File.OpenRead(TEMPORARY_BINARY);
                Stream recent = File.OpenWrite(name);

                while ( ( size = temporary.Read(buffer, 0, BLOCK_SIZE) ) > 0 )
                {
                    recent.Write(buffer, 0, size);
                }

                recent.Close();
                temporary.Close();

                if (! no_removing)
                {
                    File.Delete(TEMPORARY_BINARY);
                }

            }
            catch (IOException e240)
            {
                Error.handler( e240.ToString() );
                exit_code = FILE_ERROR_CODE;
            }
            catch (Exception e241)
            {
                Error.handler( e241.ToString() );
                exit_code = FILE_ERROR_CODE;
            }

            return exit_code;
        }

        string binary_file_in_current_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(MS932_SJIS_BIN);
                table = MS932_SJIS_BIN;
                file.Close();
            }
            catch (IOException e242)
            {
                Error.handler( e242.ToString() );
            }
            catch (Exception e243)
            {
                Error.handler( e243.ToString() );
            }

            return table;
        }

        string binary_file_in_installed_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(C_MS932_SJIS_BIN); // directory where Diyo was installed.
                table = C_MS932_SJIS_BIN;
                file.Close();
            }
            catch (IOException e244)
            {
                Error.handler( e244.ToString() );
            }
            catch (Exception e245)
            {
                Error.handler( e245.ToString() );
            }

            return table;
        }

        string binary_file_in_path_directory()
        {
            CommandPath c = new CommandPath();
            string table = c.pathName(MS932_SJIS_BIN); // directory included in Path
            return table;
        }

        string binary_file_in_selected_directory()
        {
            string table = EMPTY;

// Acquiring a table name from SETTING_TEXT
            try
            {
                string line = EMPTY;
                StreamReader sjisFile = new StreamReader(SETTING_TEXT, Encoding.Default);

                for (int i = 1; i <= FINAL_LINE; ++i)
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    switch(i)
                    {
                        case 1:
                            table = line;
                            break;
                        case FINAL_LINE:
                            table = line;
                            break;
                        default:
                            break;
                    }

                } //for (; ; )

                sjisFile.Close();
            }
            catch (IOException e246)
            {
                Error.handler( e246.ToString() );
            }
            catch (Exception e247)
            {
                Error.handler( e247.ToString() );
            }

            if (table.Length == 0) // table = EMPTY
            {
                Error.handler("binary_file_in_selected_directory() error: table.Length == 0");
            }
            else
            {
// Checking if Diyo can find the table
                try
                {
                    Stream file = File.OpenRead(table);
                    file.Close();
                }
                catch (IOException e248)
                {
                    Error.handler( e248.ToString() );
                    table = EMPTY;
                }
                catch (Exception e249)
                {
                    Error.handler( e249.ToString() );
                    table = EMPTY;
                }

            }

            return table;
        }

        string locateBinaryAssembly()
        {
            Assembly dayu = Assembly.GetExecutingAssembly();
            int q;
            string table;
            string path = dayu.Location;

            q = path.LastIndexOf(YEN);
            q += YEN_SIZE;
            table = path.Substring(0, q) + MS932_SJIS_BIN;

            // Checking if Dayu can find the table
            try
            {
                Stream file = File.OpenRead(table);
                file.Close();
            }
            catch (IOException e250)
            {
                Error.handler(e250.ToString());
                table = EMPTY;
            }
            catch (Exception e251)
            {
                Error.handler(e251.ToString());
                table = EMPTY;
            }

            return table;
        }

    }
}
