﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Dayu
{
    class SparseSJIS
    {
        const byte ERROR_CODE = 0xFF;
        const int DOUBLE_KANJI = 4; // SJIS_KANJI_SIZE + SJIS_KANJI_SIZE
        const int FIRST_KANJI = 159;
        const int FINAL_LINE = 16;
        const int RECORD_COUNT = 7882;
        const int SEARCH_COUNT = 5;
        const int SEARCH_COUNT_1 = 4; // SEARCH_COUNT - 1
        const int SJIS_KANJI_SIZE = 2;
        const int SJIS_KANJI_SIZE_1 = 3;
        const int YEN_SIZE = 1;
        const string C_MS932_SJIS_BIN = "C:\\sjisEUCjpCS\\MS932~Shift_JIS.bin";
        const string EMPTY = "";
        const string SETTING_TEXT = "twenty.txt";
        const string MS932_SJIS_BIN = "MS932~Shift_JIS.bin";
        const string YEN = "\\";

        byte[] sparse;
        delegate string OpenCloseFunction();
        int size;

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        private void populated(int index, int place, byte c)
        {
            try
            {
                int k = index * SJIS_KANJI_SIZE_1 + place;
                sparse[k] = c;
            }
            catch (IndexOutOfRangeException e234)
            {
                Error.handler( e234.ToString() );
            }
            catch (Exception e123)
            {
                Error.handler( e123.ToString() );
            }

        }

        public bool setting(SearchTable s)
        {
            byte[] c = {0, 0, 0, 0};
            int beginning = 0;
            int index;
            OpenCloseFunction[] openClose = new OpenCloseFunction[SEARCH_COUNT];
            string table = EMPTY;

            openClose[0] = binary_file_in_installed_directory;
            openClose[1] = binary_file_in_current_directory;
            openClose[2] = binary_file_in_path_directory;
            openClose[3] = locateBinaryAssembly;
            openClose[4] = binary_file_in_selected_directory;

            if ( s.priority() )
            {
                beginning = SEARCH_COUNT_1;
            }

            for (int i = beginning; i < SEARCH_COUNT; ++i)
            {
                table = openClose[i]();
                if ( table.Length > 0 )
                {
                    if ( s.whichTable() )
                    {
                        Console.WriteLine( table + " is primarily accessible to Dayu." );
                        s.whichTable(false);
                    }

                    break;
                }
                else
                {
                    switch(i)
                    {
                        case 0:
                            if( s.noSearching() ) {
                                Console.Error.Write("openClose[");
                                Console.Error.Write(i);
                                Console.Error.Write("]() error: ");
                                Console.Error.WriteLine(MS932_SJIS_BIN);
                                return false;
                            }

                            break;
                        case SEARCH_COUNT_1:
                            Console.Error.Write("openClose[");
                            Console.Error.Write(i);
                            Console.Error.Write("]() error: ");
                            Console.Error.WriteLine(MS932_SJIS_BIN);
                            return false;
                        default:
                            break;
                    }

                }

            }

            Stream binaryFile = File.OpenRead(table);
            for(int i = 1; i <= RECORD_COUNT; ++i)
            {
                binaryFile.Read(c, 0, DOUBLE_KANJI);
                if (i < FIRST_KANJI)
                {
                    index = (int)c[0];
                    populated(index, 0, c[SJIS_KANJI_SIZE]);
                }
                else
                {
                    populated( inverted(c), 0, c[SJIS_KANJI_SIZE]);
                    populated( inverted(c), 1, c[ SJIS_KANJI_SIZE + 1 ]);
                }
 
            }

            binaryFile.Close();
            return true;
        }

        public void clear()
        {
            for(int i = 0; i < sparse.Length; ++i)
            {
                sparse[i] = 0;
            }

        }

        public SparseSJIS()
        {
            size = 0xFC4C * SJIS_KANJI_SIZE_1;
            sparse = new byte[size];
        }

        public byte working(int index, int place)
        {
            int k = index * SJIS_KANJI_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e124)
            {
                Error.handler( e124.ToString() );
                Console.Write("sparse[] error: ");
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }
            catch (Exception e125)
            {
                Error.handler( e125.ToString() );
                r = ERROR_CODE;
            }

            return r;
        }

        string binary_file_in_current_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(MS932_SJIS_BIN);
                table = MS932_SJIS_BIN;
                file.Close();
            }
            catch (IOException e126)
            {
                Error.handler( e126.ToString() );
            }
            catch (Exception e127)
            {
                Error.handler( e127.ToString() );
            }

            return table;
        }

        string binary_file_in_installed_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(C_MS932_SJIS_BIN); // directory where Dayu was installed.
                table = C_MS932_SJIS_BIN;
                file.Close();
            }
            catch (IOException e128)
            {
                Error.handler( e128.ToString() );
            }
            catch (Exception e129)
            {
                Error.handler( e129.ToString() );
            }

            return table;
        }

        string binary_file_in_path_directory()
        {
            CommandPath c = new CommandPath();
            string table = c.pathName(MS932_SJIS_BIN); // directory included in Path
            return table;
        }

        string binary_file_in_selected_directory()
        {
            string table = EMPTY;

// Acquiring a table name from SETTING_TEXT
            try
            {
                string line = EMPTY;
                StreamReader sjisFile = new StreamReader(SETTING_TEXT, Encoding.Default);

                for (int i = 1; i <= FINAL_LINE; ++i)
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    switch(i)
                    {
                        case 1:
                            table = line;
                            break;
                        case FINAL_LINE:
                            table = line;
                            break;
                        default:
                            break;
                    }

                } //for (; ; )

                sjisFile.Close();
            }
            catch (IOException e130)
            {
                Error.handler( e130.ToString() );
            }
            catch (Exception e131)
            {
                Error.handler( e131.ToString() );
            }

            if (table.Length == 0) // table = EMPTY
            {
                Error.handler("binary_file_in_selected_directory() error: table.Length == 0");
            }
            else
            {
// Checking if Dayu can find the table
                try
                {
                    Stream file = File.OpenRead(table);
                    file.Close();
                }
                catch (IOException e132)
                {
                    Error.handler( e132.ToString() );
                    table = EMPTY;
                }
                catch (Exception e133)
                {
                    Error.handler( e133.ToString() );
                    table = EMPTY;
                }

            }

            return table;
        }

        string locateBinaryAssembly()
        {
            Assembly dayu = Assembly.GetExecutingAssembly();
            int q;
            string table;
            string path = dayu.Location;

            q = path.LastIndexOf(YEN);
            q += YEN_SIZE;
            table = path.Substring(0, q) + MS932_SJIS_BIN;

            // Checking if Dayu can find the table
            try
            {
                Stream file = File.OpenRead(table);
                file.Close();
            }
            catch (IOException e166)
            {
                Error.handler(e166.ToString());
                table = EMPTY;
            }
            catch (Exception e162)
            {
                Error.handler(e162.ToString());
                table = EMPTY;
            }

            return table;
        }

    }
}
