﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/*
Diyo C# Version 1.03
Editing MS932~EUC-JP.bin
Copyright 2019 Sogaya

Visual C# 2008 Express
Visual Studio 2010

Diyo assumes that the little-endian architecture underlies operating systems (Linux, Windows).
*/

namespace Diyo
{
    class Program
    {
        static void Main(string[] args)
        {
            const bool WITH_COMMENTS = true;
            const bool WITHOUT_COMMENTS = false;
            const double VERSION = 1.03;
            const int ERROR_CODE = 1;
            const string ACQUIRE_TABLE_NAME_FROM_TEXT = "-q";
            const string ASCII = "-a";
            const string DO_NOT_SEARCH_TABLE = "-d";
            const string DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE = "-w-";
            const string EMPTY = "";
            const string EUC_JP = "-e";
            const string KATAKANA = "-k";
            const string LIST_FILE = "-f";
            const string SEARCH_TABLE = "-d-";
            const string SETTING_TEXT = "setting.txt";
            const string SHIFT_JIS = "-s";
            const string SJIS_EUC_BIN = "MS932~EUC-JP.bin";
            const string VERSION_WITH_COMMENTS = "-VC";
            const string VERSION_WITHOUT_COMMENTS = "-V";
            const string WHICH_TABLE = "-w";

            bool file_to_be_read = false;
            bool no_removing = false;
            bool no_searching = false;
            bool result = true;
            bool selected_directory_has_priority = false;
            bool which_table = false;
            Guide g = new Guide();
            int exit_code = 0;
            string binaryName = SJIS_EUC_BIN;
            string file_name = SJIS_EUC_BIN;
            string head = EMPTY;
            string text_of_substitutes_list = EMPTY;
            string normal = "⑳"; // 8753h
            string substitute = "廿"; // 93F9h

// Acquiring parameters from SETTING_TEXT unless the user removes it.
            AcquireFromText a = new AcquireFromText(SETTING_TEXT, file_name, binaryName, normal, substitute, text_of_substitutes_list, no_searching, which_table);
            a.acquiring();
            file_name = a.fileName();
            binaryName = a.newName();
            normal = a.normalCharacter();
            substitute = a.substituteCharacter();
            text_of_substitutes_list = a.substitutesList();
            no_searching = a.noSearching();
            which_table = a.whichTable();

            foreach (string argument in args)
            {
                switch (argument)
                {
                    case ACQUIRE_TABLE_NAME_FROM_TEXT:
                        selected_directory_has_priority = true;
                        break;
                    case DO_NOT_SEARCH_TABLE:
                        no_searching = true;
                        break;
                    case DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE:
                        which_table = false;
                        break;
                    case LIST_FILE:
                        file_to_be_read = true;
                        break;
                    case SEARCH_TABLE:
                        no_searching = false;
                        break;
                    case VERSION_WITH_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITH_COMMENTS);
                        System.Environment.Exit(exit_code);
                        break;
                    case VERSION_WITHOUT_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITHOUT_COMMENTS);
                        System.Environment.Exit(exit_code);
                        break;
                    case WHICH_TABLE:
                        which_table = true;
                        break;
                    default:
                        head = argument.Substring(0, 2);
                        switch (head)
                        {
                            case ASCII:
                                substitute = argument.Substring(2, 1);
                                file_to_be_read = false;
                                break;
                            case EUC_JP:
                                substitute = argument.Substring(2, 1);
                                file_to_be_read = false;
                                break;
                            case KATAKANA:
                                normal = argument.Substring(2, 1);
                                file_to_be_read = false;
                                break;
                            case SHIFT_JIS:
                                normal = argument.Substring(2, 1);
                                file_to_be_read = false;
                                break;
                            default:
                                if (file_to_be_read)
                                {
                                    text_of_substitutes_list = argument;
                                    file_to_be_read = false;
                                }
                                else
                                {
                                    binaryName = argument;
                                }

                                break;
                        } // switch (head)

                        break;
                } // switch (argument)

            } // foreach (string argument in args)

            Editing e = new Editing();
            SearchTable s = new SearchTable(binaryName, no_searching, selected_directory_has_priority, which_table);

            if (text_of_substitutes_list.Length == 0)
            {
                result = e.editRecord(s, normal, substitute, binaryName, no_removing);
            }
            else
            {
                result = e.editManyRecords(s, text_of_substitutes_list, binaryName, no_removing);
            }

            exit_code = result ? 0 : ERROR_CODE;
            System.Environment.Exit(exit_code);
        }

    }
}
