﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Diyo
{
    class SparseEUC
    {
        const byte ERROR_CODE = 0xFF;
        const int DOUBLE_KANJI = 4; // SJIS_KANJI_SIZE + EUC_KANJI_SIZE
        const int EUC_KANJI_SIZE = 2;
        const int EUC_KANJI_SIZE_1 = 3;
        const int FIRST_KANJI = 159;
        const int FINAL_LINE = 7;
        const int RECORD_COUNT = 7882;
        const int SEARCH_COUNT = 5;
        const int SEARCH_COUNT_1 = 4; // SEARCH_COUNT - 1
        const int SJIS_KANJI_SIZE = 2;
        const int YEN_SIZE = 1;
        const string C_SJIS_EUC_BIN = "C:\\sjisEUCjpCS\\MS932~EUC-JP.bin";
        const string EMPTY = "";
        const string SETTING_TEXT = "setting.txt";
        const string SJIS_EUC_BIN = "MS932~EUC-JP.bin";
        const string YEN = "\\";

        byte[] sparse;
        delegate string OpenCloseFunction();
        int size;

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        private void populated(int index, int place, byte c)
        {
            try
            {
                int k = index * EUC_KANJI_SIZE_1 + place;
                sparse[k] = c;
            }
            catch (IndexOutOfRangeException e22)
            {
                Error.handler( e22.ToString() );
            }
            catch (Exception e23)
            {
                Error.handler( e23.ToString() );
            }

        }

        public bool setting(SearchTable s)
        {
            byte[] c = {0, 0, 0, 0};
            int beginning = 0;
            int index;
            OpenCloseFunction[] openClose = new OpenCloseFunction[SEARCH_COUNT];
            string table = EMPTY;

            openClose[0] = binary_file_in_installed_directory;
            openClose[1] = binary_file_in_current_directory;
            openClose[2] = binary_file_in_path_directory;
            openClose[3] = locateBinaryAssembly;
            openClose[4] = binary_file_in_selected_directory;

            if ( s.priority() )
            {
                beginning = SEARCH_COUNT_1;
            }

            for (int i = beginning; i < SEARCH_COUNT; ++i)
            {
                table = openClose[i]();
                if ( table.Length > 0 )
                {
                    if ( s.whichTable() )
                    {
                        Console.WriteLine( table + " is primarily accessible to Diyo." );
                        s.whichTable(false);
                    }

                    break;
                }
                else
                {
                    switch(i)
                    {
                        case 0:
                            if( s.noSearching() ) {
                                Console.Error.Write("openClose[");
                                Console.Error.Write(i);
                                Console.Error.Write("]() error: ");
                                Console.Error.WriteLine(SJIS_EUC_BIN);
                                return false;
                            }

                            break;
                        case SEARCH_COUNT_1:
                            Console.Error.Write("openClose[");
                            Console.Error.Write(i);
                            Console.Error.Write("]() error: ");
                            Console.Error.WriteLine(SJIS_EUC_BIN);
                            return false;
                        default:
                            break;
                    }

                }

            }

            Stream binaryFile = File.OpenRead(table);
            for(int i = 1; i <= RECORD_COUNT; ++i)
            {
                binaryFile.Read(c, 0, DOUBLE_KANJI);
                if (i < FIRST_KANJI)
                {
                    index = (int)c[0];
                    populated(index, 0, c[SJIS_KANJI_SIZE]);
                }
                else
                {
                    populated( inverted(c), 0, c[SJIS_KANJI_SIZE]);
                    populated( inverted(c), 1, c[ SJIS_KANJI_SIZE + 1 ]);
                }
 
            }

            binaryFile.Close();
            return true;
        }

        public void clear()
        {
            for(int i = 0; i < sparse.Length; ++i)
            {
                sparse[i] = 0;
            }

        }

        public SparseEUC()
        {
            size = 0xFC4C * EUC_KANJI_SIZE_1;
            sparse = new byte[size];
        }

        public byte working(int index, int place)
        {
            int k = index * EUC_KANJI_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e24)
            {
                Error.handler( e24.ToString() );
                Console.Write("sparse[] error: ");
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }
            catch (Exception e25)
            {
                Error.handler( e25.ToString() );
                r = ERROR_CODE;
            }

            return r;
        }

        string binary_file_in_current_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(SJIS_EUC_BIN);
                table = SJIS_EUC_BIN;
                file.Close();
            }
            catch (IOException e26)
            {
                Error.handler( e26.ToString() );
            }
            catch (Exception e27)
            {
                Error.handler( e27.ToString() );
            }

            return table;
        }

        string binary_file_in_installed_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(C_SJIS_EUC_BIN); // directory where Diyo was installed.
                table = C_SJIS_EUC_BIN;
                file.Close();
            }
            catch (IOException e28)
            {
                Error.handler( e28.ToString() );
            }
            catch (Exception e29)
            {
                Error.handler( e29.ToString() );
            }

            return table;
        }

        string binary_file_in_path_directory()
        {
            CommandPath c = new CommandPath();
            string table = c.pathName(SJIS_EUC_BIN); // directory included in Path
            return table;
        }

        string binary_file_in_selected_directory()
        {
            string table = EMPTY;

// Acquiring a table name from SETTING_TEXT
            try
            {
                string line = EMPTY;
                StreamReader sjisFile = new StreamReader(SETTING_TEXT, Encoding.Default);

                for (int i = 1; i <= FINAL_LINE; ++i)
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    switch(i)
                    {
                        case 1:
                            table = line;
                            break;
                        case FINAL_LINE:
                            table = line;
                            break;
                        default:
                            break;
                    }

                } //for (; ; )

                sjisFile.Close();
            }
            catch (IOException e30)
            {
                Error.handler( e30.ToString() );
            }
            catch (Exception e31)
            {
                Error.handler( e31.ToString() );
            }

            if (table.Length == 0) // table = EMPTY
            {
                Error.handler("binary_file_in_selected_directory() error: table.Length == 0");
            }
            else
            {
// Checking if Diyo can find the table
                try
                {
                    Stream file = File.OpenRead(table);
                    file.Close();
                }
                catch (IOException e32)
                {
                    Error.handler( e32.ToString() );
                    table = EMPTY;
                }
                catch (Exception e33)
                {
                    Error.handler( e33.ToString() );
                    table = EMPTY;
                }

            }

            return table;
        }

        string locateBinaryAssembly()
        {
            Assembly diyo = Assembly.GetExecutingAssembly();
            int q;
            string table;
            string path = diyo.Location;

            q = path.LastIndexOf(YEN);
            q += YEN_SIZE;
            table = path.Substring(0, q) + SJIS_EUC_BIN;

            // Checking if Diyo can find the table
            try
            {
                Stream file = File.OpenRead(table);
                file.Close();
            }
            catch (IOException e171)
            {
                Error.handler(e171.ToString());
                table = EMPTY;
            }
            catch (Exception e172)
            {
                Error.handler(e172.ToString());
                table = EMPTY;
            }

            return table;
        }

    }
}
