﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Hanzen
{
    class DividingEditing : Dividing
    {
        const int ASCII_SIZE = 1;
        const int EDIT_FUNCTION_FACTOR = 2;
        const int HA_HI_HU_HE_HO = 15;
        const int KANA_DAKUTEN = 0xDE;
        const int KANA_HANDAKUTEN = 0xDF;
        const int KANA_KUTEN = 0xA1;
        const int KATAKANA_SIZE = 1;
        const int ZENKAKU_SIZE = 2;

        SparseKana kana;

        private bool seion(int beginning, byte first_byte, byte second_byte)
        {
            bool b = false;
            byte[] d = {0, 0};
            int[] ka_sa_ta_ha = {0x834A, 0x834C, 0x834E, 0x8350, 0x8352, 0x8354, 0x8356, 0x8358, 0x835A, 0x835C, 0x835E, 0x8360, 0x8363, 0x8365, 0x8367, 0x836E, 0x8371, 0x8374, 0x8377, 0x837A};

            d[0] = first_byte;
            d[1] = second_byte;
            int test = inverted(d);

            for (int i = beginning; i < ka_sa_ta_ha.Length; ++i)
            {
                if (ka_sa_ta_ha[i] == test)
                {
                    b = true;
                    break;
                }

            }

            return b;
        }

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        public DividingEditing(FieldsList fields_list, char s, byte c, bool e, SparseKana k) : base(fields_list, s, c, e)
        {
            kana = k;
        }

        public DividingEditing(FieldsList fields_list, int c, char s, bool e, SparseKana k)
            : base(fields_list, c, s, e)
        {
            kana = k;
        }

        public override int editing(string p, int field_number)
        {
            smashed_to_pieces(p);

            vector_to_array_editing(hankaku_zenkaku, field_number);
            return field_count;
        }

        string hankaku_zenkaku(string field)
        {
            bool kanji = false;
            byte firstByte = 0x40;
            int c;
            int previous = 0xB6;
            int k = 0;

            try
            {
                Encoding MS932 = Encoding.GetEncoding(932);
                byte[] buffer = MS932.GetBytes(field);
                int size = MS932.GetByteCount(field);
                byte[] combo = new byte[ size * EDIT_FUNCTION_FACTOR ];

                for (int i = 0; i < size; ++i)
                {
                    c = (int)buffer[i];

                    if (kanji)
                    {
                        combo[k] = firstByte;
                        combo[ k + 1 ] = buffer[i];
                        k += ZENKAKU_SIZE;
                        kanji = false;
                        continue;
                    }

                    if (0x81 <= c && c <= 0xFC)
                    {
                        if (KANA_KUTEN <= c && c < KANA_DAKUTEN)
                        {
                            combo[k] = kana.working(c, 0);
                            combo[ k + 1 ] = kana.working(c, 1);
                            k += ZENKAKU_SIZE;
                            previous = c;
                        }
                        else if (c == KANA_DAKUTEN)
                        {
                            // combining character
                            k -= ZENKAKU_SIZE;

                            if ( seion(0, combo[k], combo[ k + 1 ]) )
                            {
                                combo[k] = kana.dakuten(previous, 0);
                                combo[ k + 1 ] = kana.dakuten(previous, 1);
                                k += ZENKAKU_SIZE;
                            }
                            else
                            {
                                k += ZENKAKU_SIZE;
                            }

                        }
                        else if (c == KANA_HANDAKUTEN)
                        {
                            // combining character
                            k -= ZENKAKU_SIZE;

                            if ( seion(HA_HI_HU_HE_HO, combo[k], combo[ k + 1 ]) )
                            {
                                combo[k] = kana.handakuten(previous, 0);
                                combo[ k + 1 ] = kana.handakuten(previous, 1);
                                k += ZENKAKU_SIZE;
                            }
                            else
                            {
                                k += ZENKAKU_SIZE;
                            }

                        }
                        else
                        {
                            firstByte = buffer[i];
                            kanji = true;
                        }

                    }
                    else
                    {
                        combo[k] = buffer[i];
                        k += ASCII_SIZE;
                    }

                } // for (int i = 0; i < size; ++i)

                field = MS932.GetString(combo);
                k = field.IndexOf('\0');
                if (k >= 0)
                {
                    field = field.Substring(0, k);
                }

            }
            catch (ArgumentException e187)
            {
                Error.handler(e187.ToString());
            }
            catch (Exception e188)
            {
                Error.handler(e188.ToString());
            }

            return field;
        }

    }
}
