﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Hanzen
{
    class SparseKana
    {
        const bool DAKUTEN = false;
        const bool HANDAKUTEN = true;
        const byte ERROR_CODE = 0xFF;
        const int DAKUTEN_OFFSET = 0x100;
        const int FINAL_LINE = 10; // Please see AcquireFromText.cs
        const int FIRST_DAKUTEN = 62;
        const int FIRST_HANDAKUTEN = 82;
        const int HANDAKUTEN_OFFSET = 0x200;
        const int HANKAKU_ZENKAKU = 3; // KATAKANA_SIZE + ZENKAKU_SIZE
        const int KATAKANA_SIZE = 1;
        const int RECORD_COUNT = 86;
        const int SEARCH_COUNT = 5;
        const int SEARCH_COUNT_1 = 4; // SEARCH_COUNT - 1
        const int YEN_SIZE = 1;
        const int ZENKAKU_SIZE = 2;
        const int ZENKAKU_SIZE_1 = 3;
        const string C_HANZEN_BINARY = "C:\\sjisEUCjpCS\\Hankaku~Zenkaku_katakana.bin";
        const string EMPTY = "";
        const string HANZEN_BINARY = "Hankaku~Zenkaku_katakana.bin";
        const string SETTING_TEXT = "hanzense.txt";
        const string YEN = "\\";

        byte[] sparse;
        delegate string OpenCloseFunction();
        int size;

        private int moved(byte c, bool handakuten)
        {
            int offset = handakuten ? HANDAKUTEN_OFFSET : DAKUTEN_OFFSET;
            int index = (int)c + offset;
            return index;
        }

        private void populated(int index, int place, byte c)
        {
            try
            {
                int k = index * ZENKAKU_SIZE_1 + place;
                sparse[k] = c;
            }
            catch (IndexOutOfRangeException e206)
            {
                Error.handler(e206.ToString());
            }
            catch (Exception e207)
            {
                Error.handler(e207.ToString());
            }

        }

        public bool setting(SearchTable s)
        {
            byte[] c = {0, 0, 0, 0};
            int beginning = 0;
            int index;
            OpenCloseFunction[] openClose = new OpenCloseFunction[SEARCH_COUNT];
            string table = EMPTY;

            openClose[0] = binary_file_in_installed_directory;
            openClose[1] = binary_file_in_current_directory;
            openClose[2] = binary_file_in_path_directory;
            openClose[3] = locateBinaryAssembly;
            openClose[4] = binary_file_in_selected_directory;

            if ( s.priority() )
            {
                beginning = SEARCH_COUNT_1;
            }

            for (int i = beginning; i < SEARCH_COUNT; ++i)
            {
                table = openClose[i]();
                if ( table.Length > 0 )
                {
                    if ( s.whichTable() )
                    {
                        Console.WriteLine( table + " is primarily accessible to Hanzen." );
                        s.whichTable(false);
                    }

                    break;
                }
                else
                {
                    switch(i)
                    {
                        case 0:
                            if( s.noSearching() ) {
                                Console.Error.Write("openClose[");
                                Console.Error.Write(i);
                                Console.Error.Write("]() error: ");
                                Console.Error.WriteLine(HANZEN_BINARY);
                                return false;
                            }

                            break;
                        case SEARCH_COUNT_1:
                            Console.Error.Write("openClose[");
                            Console.Error.Write(i);
                            Console.Error.Write("]() error: ");
                            Console.Error.WriteLine(HANZEN_BINARY);
                            return false;
                        default:
                            break;
                    }

                }

            }

            Stream binaryFile = File.OpenRead(table);
            for(int i = 1; i <= RECORD_COUNT; ++i)
            {
                binaryFile.Read(c, 0, HANKAKU_ZENKAKU);
                if(i < FIRST_DAKUTEN)
                {
                    index = (int)c[0];
                    populated(index, 0, c[KATAKANA_SIZE]);
                    populated(index, 1, c[ KATAKANA_SIZE + 1 ]);
                }
                else if (i >= FIRST_HANDAKUTEN)
                {
                    populated( moved(c[0], HANDAKUTEN), 0, c[KATAKANA_SIZE]);
                    populated( moved(c[0], HANDAKUTEN), 1, c[ KATAKANA_SIZE + 1 ]);
                }
                else
                {
                    populated( moved(c[0], DAKUTEN), 0, c[KATAKANA_SIZE]);
                    populated( moved(c[0], DAKUTEN), 1, c[ KATAKANA_SIZE + 1 ]);
                }

            }

            binaryFile.Close();
            return true;
        }

        public void clear()
        {
            for(int i = 0; i < sparse.Length; ++i)
            {
                sparse[i] = 0;
            }

        }

        public SparseKana()
        {
            size = 0x02E0 * ZENKAKU_SIZE_1;
            sparse = new byte[size];
        }

        public byte dakuten(int index, int place)
        {
            int k = ( index + DAKUTEN_OFFSET) * ZENKAKU_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e208)
            {
                Error.handler(e208.ToString());
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }

            return r;
        }

        public byte handakuten(int index, int place)
        {
            int k = ( index + HANDAKUTEN_OFFSET) * ZENKAKU_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e209)
            {
                Error.handler(e209.ToString());
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }

            return r;
        }

        public byte working(int index, int place)
        {
            int k = index * ZENKAKU_SIZE_1 + place;
            byte r = 0;

            try
            {
                r = sparse[k];
            }
            catch (IndexOutOfRangeException e210)
            {
                Error.handler(e210.ToString());
                Console.WriteLine( string.Format("{0:X4}", k) );
                r = ERROR_CODE;
            }

            return r;
        }

        string binary_file_in_current_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(HANZEN_BINARY);
                table = HANZEN_BINARY;
                file.Close();
            }
            catch (IOException e211)
            {
                Error.handler( e211.ToString() );
            }
            catch (Exception e212)
            {
                Error.handler( e212.ToString() );
            }

            return table;
        }

        string binary_file_in_installed_directory()
        {
            string table = EMPTY;

            try
            {
                Stream file = File.OpenRead(C_HANZEN_BINARY); // directory where hanzen was installed.
                table = C_HANZEN_BINARY;
                file.Close();
            }
            catch (IOException e213)
            {
                Error.handler( e213.ToString() );
            }
            catch (Exception e214)
            {
                Error.handler( e214.ToString() );
            }

            return table;
        }

        string binary_file_in_path_directory()
        {
            CommandPath c = new CommandPath();
            string table = c.pathName(HANZEN_BINARY); // directory included in Path
            return table;
        }

        string binary_file_in_selected_directory()
        {
            string table = EMPTY;

// Acquiring a table name from SETTING_TEXT
            try
            {
                string line = EMPTY;
                StreamReader sjisFile = new StreamReader(SETTING_TEXT, Encoding.Default);

                for (int i = 1; i <= FINAL_LINE; ++i)
                {
                    line = sjisFile.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    switch(i)
                    {
                        case 1:
                            table = line;
                            break;
                        case FINAL_LINE:
                            table = line;
                            break;
                        default:
                            break;
                    }

                } //for (; ; )

                sjisFile.Close();
            }
            catch (IOException e215)
            {
                Error.handler( e215.ToString() );
            }
            catch (Exception e216)
            {
                Error.handler( e216.ToString() );
            }

            if (table.Length == 0) // table = EMPTY
            {
                Error.handler("binary_file_in_selected_directory() error: table.Length == 0");
            }
            else
            {
// Checking if hanzen can find the table
                try
                {
                    Stream file = File.OpenRead(table);
                    file.Close();
                }
                catch (IOException e217)
                {
                    Error.handler( e217.ToString() );
                    table = EMPTY;
                }
                catch (Exception e218)
                {
                    Error.handler( e218.ToString() );
                    table = EMPTY;
                }

            }

            return table;
        }

        string locateBinaryAssembly()
        {
            Assembly hanzen = Assembly.GetExecutingAssembly();
            int q;
            string binaryName, table;
            string path = hanzen.Location;

            q = path.LastIndexOf(YEN);
            q += YEN_SIZE;
            binaryName = path.Substring(0, q) + HANZEN_BINARY;

            // Checking if Hanzen can find the table
            try
            {
                Stream file = File.OpenRead(binaryName);
                file.Close();
                table = binaryName;
            }
            catch (IOException e219)
            {
                Error.handler(e219.ToString());
                table = EMPTY;
            }
            catch (Exception e220)
            {
                Error.handler(e220.ToString());
                table = EMPTY;
            }

            return table;
        }

    }
}
