﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;

/*
Hetn Version 1.01
Reading sutai.xml to arrange lines in descending order, for example
Copyright 2019 Sogaya

Visual C# 2008 Express
Visual Studio 2010

*/

namespace Hetn
{
    class Program
    {
        static void Main(string[] args)
        {
            const bool NO_REMOVING = true;
            const bool WITH_COMMENTS = true;
            const bool WITHOUT_COMMENTS = false;
            const double VERSION = 1.01;
            const char EXCLAMATION_MARK = '!';
            const int BUFFER_SIZE = 524288;
            const int BYTE_JOINT = 2;
            const int EUC_JP_ENCODING = 1;
            const int EXCLAMATION_JOINT = 0;
            const int FLOATING_POINT_NUMBERS = 2;
            const int INTEGERS = 1;
            const int NIBBLE_JOINT = 1;
            const int PHRASES = 0;
            const int SHIFT_JIS_ENCODING = 2;
            const int UTF_8_ENCODING = 3;
            const string AMPERSAND = "-A";
            const string ASCENDING = "-r-";
            const string BOTH = "-N";
            const string BRACKETS = "-B";
            const string CASE_SENSITIVE = "-f-";
            const string COUNT_TICKS = "--tick";
            const string CRLF = "-CRLF";
            const string DESCENDING = "-r";
            const string DO_NOT_COUNT_TICKS = "-T-";
            const string DO_NOT_REMOVE_0DH = "-L-";
            const string DO_NOT_REMOVE_TEMPORARY_FILES = "-y";
            const string DO_NOT_REPLACE_AMPERSAND = "-A-";
            const string EMPTY = "";
            const string EUC_JP_TERM = "-e";
            const string HELP = "--help";
            const string INTERMEDIATE_FILE = "intermediate.tmp";
            const string INTERMEDIATE_FILE_2 = "intermediate-2.tmp";
            const string INTERMEDIATE_FILE_3 = "intermediate-3.tmp";
            const string LF = "-LF";
            const string LINE_FEED = "-L";
            const string NEW_FILE = "-o";
            const string NOT_CASE_SENSITIVE = "-f";
            const string QUICK_CHECK = "-S-";
            const string REPORT_ID = "-!";
            const string SETTING_TEXT = "manager.txt";
            const string SILENT = "-s";
            const string STANDARD_OUT = "-o-";
            const string STRICT = "-S";
            const string SUTAI_XML = "sutai.xml";
            const string TEMPORARY_BINARY = "binary.tmp";
            const string TICK = "-T";
            const string UTF_8_TERM = "-8";
            const string VERSION_WITH_COMMENTS = "-VC";
            const string VERSION_WITHOUT_COMMENTS = "-V";
            const string XML_TERM = "-B-";

            bool add_ID = false;
            bool brackets = false;
            bool carriage_return = true;
            bool case_sensitive = true;
            bool descending = false;
            bool insert_both_bytes_and_nibbles = false;
            bool loud = true;
            bool new_file = false;
            bool remove_temporary_files = true;
            bool replace_ampersand = true;
            bool report_ID = false;
            bool strictly_check = false;
            bool ticktock = false;
            char joint = EXCLAMATION_MARK;
            Encoding MS932 = Encoding.GetEncoding(932);
            Encoding encoded = MS932;
            Encoding EUC_JP = Encoding.GetEncoding("euc-jp");
            Encoding UTF_8 = Encoding.UTF8;
            Guide g = new Guide();
            int encoding = SHIFT_JIS_ENCODING;
            int exit_code = 0;
            int joint_type = EXCLAMATION_JOINT; // 0: EXCLAMATION_JOINT. 1: NIBBLE_JOINT. 2: BYTE_JOINT
            int line_count = 0;
            int manager = PHRASES;
            Stopwatch w = Stopwatch.StartNew();
            string xml_name = SUTAI_XML;
            string new_name = EMPTY;

// Acquiring parameters from SETTING_TEXT unless the user removes it.
            AcquireFromText t = new AcquireFromText(SETTING_TEXT, new_name, xml_name, encoding, joint_type, line_count, manager, joint, add_ID, brackets, carriage_return, case_sensitive, descending, insert_both_bytes_and_nibbles, loud, replace_ampersand, report_ID, strictly_check);
            t.acquiring();
            add_ID = t.addID();
            brackets = t.lessThanGreaterThan();
            carriage_return = t.carriageReturn();
            case_sensitive = t.caseSensitive();
            descending = t.descendingOrder();
            encoding = t.encodingMethod();
            insert_both_bytes_and_nibbles = t.bytesNibbles();
            joint = t.jointValue();
            joint_type = t.jointType();
            line_count = t.lineCount();
            loud = t.notSilent();
            manager = t.management();
            new_name = t.newName();
            replace_ampersand = t.substituteForAmpersand();
            report_ID = t.reportID();
            strictly_check = t.strictFunction();
            xml_name = t.XML();

            switch(encoding)
            {
                case EUC_JP_ENCODING:
                    encoded = EUC_JP;
                    break;
                case SHIFT_JIS_ENCODING:
                    encoded = MS932;
                    break;
                case UTF_8_ENCODING:
                    encoded = UTF_8;
                    break;
            }

            foreach (string s in args)
            {
                switch (s)
                {
                    case AMPERSAND:
                        replace_ampersand = true;
                        break;
                    case ASCENDING:
                        descending = false;
                        break;
                    case BOTH:
                        insert_both_bytes_and_nibbles = true;
                        break;
                    case BRACKETS:
                        brackets = true;
                        break;
                    case CASE_SENSITIVE:
                        case_sensitive = true;
                        break;
                    case COUNT_TICKS:
                        ticktock = true;
                        break;
                    case CRLF:
                        carriage_return = true;
                        break;
                    case DESCENDING:
                        descending = true;
                        break;
                    case DO_NOT_COUNT_TICKS:
                        ticktock = false;
                        break;
                    case DO_NOT_REMOVE_0DH:
                        carriage_return = true;
                        break;
                    case DO_NOT_REMOVE_TEMPORARY_FILES:
                        remove_temporary_files = false;
                        break;
                    case DO_NOT_REPLACE_AMPERSAND:
                        replace_ampersand = false;
                        break;
                    case EUC_JP_TERM:
                        encoded = EUC_JP;
                        break;
                    case HELP:
                        exit_code = g.versionGuide(VERSION, WITH_COMMENTS, loud);
                        System.Environment.Exit(exit_code);
                        break;
                    case LF:
                        carriage_return = false;
                        break;
                    case LINE_FEED:
                        carriage_return = false;
                        break;
                    case NEW_FILE:
                        new_file = true;
                        break;
                    case NOT_CASE_SENSITIVE:
                        case_sensitive = false;
                        break;
                    case QUICK_CHECK:
                        strictly_check = false;
                        break;
                    case REPORT_ID:
                        report_ID = true;
                        break;
                    case SILENT:
                        loud = false;
                        break;
                    case STANDARD_OUT:
                        new_name = EMPTY;
                        break;
                    case STRICT:
                        strictly_check = true;
                        break;
                    case TICK:
                        ticktock = true;
                        break;
                    case UTF_8_TERM:
                        encoded = UTF_8;
                        break;
                    case VERSION_WITH_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITH_COMMENTS, loud);
                        System.Environment.Exit(exit_code);
                        break;
                    case VERSION_WITHOUT_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITHOUT_COMMENTS, loud);
                        System.Environment.Exit(exit_code);
                        break;
                    case XML_TERM:
                        brackets = false;
                        break;
                    default:
                        if (new_file)
                        {
                            new_name = s;
                            new_file = false;
                        }
                        else
                        {
                            xml_name = s;
                        }

                        break;
                } // switch (s)

            } // foreach (string s in args)

            if (! ticktock)
            {
                w.Stop();
            }

            Intermediate f = new Intermediate(SUTAI_XML, BUFFER_SIZE);

            try
            {
// Inserting LF to produce INTERMEDIATE_FILE
                if (strictly_check)
                {
                    f.insert_LF_strictly();
                }
                else
                {
                    f.insertLF();
                }

// Replacing to produce TEMPORARY_BINARY
                if (replace_ampersand)
                {
                    f.ampersand();
                }
                else
                {
                    f.produceBinaryFile(INTERMEDIATE_FILE, NO_REMOVING);
                }

                if (brackets)
                {
                    f.lessThan(); // Reading TEMPORARY_BINARY to produce INTERMEDIATE_FILE_2
                    f.greaterThan(); // Reading INTERMEDIATE_FILE_2 to produce TEMPORARY_BINARY
                }

                switch (joint_type)
                {
                    case BYTE_JOINT:
// Inserting a joint to produce INTERMEDIATE_FILE_3
                        f.hexadecimal();
                        f.produceBinaryFile(INTERMEDIATE_FILE_3, NO_REMOVING);
                        break;
                    case NIBBLE_JOINT:
                        f.nibble();
                        f.produceBinaryFile(INTERMEDIATE_FILE_3, NO_REMOVING);
                        break;
                    default:
                        if (insert_both_bytes_and_nibbles)
                        {
                            f.hexadecimal();
                            f.inserting_nibbles_to_produce_binary_file();
                        }

                        break;
                }

            }
            catch (Exception e513)
            {
                Error.handler( e513.ToString() );
                System.Environment.Exit(exit_code);
            }

            if (descending)
            {
                DescendingOrder d = new DescendingOrder(encoded, new_name, TEMPORARY_BINARY, line_count, joint, add_ID, carriage_return, report_ID);
                switch (manager)
                {
                    case PHRASES:
                        if (case_sensitive)
                        {
                            exit_code = d.arranging_to_produce_text();
                        }
                        else
                        {
                            exit_code = d.case_insensitive_arranging_to_produce_text();
                        }

                        break;
                    case INTEGERS:
                        exit_code = d.arranging_integers_to_produce_text();
                        break;
                    case FLOATING_POINT_NUMBERS:
                        exit_code = d.arranging_doubles_to_produce_text();
                        break;
                    default:
                        break;
                }

            }
            else
            {
                AscendingOrder a = new AscendingOrder(encoded, new_name, TEMPORARY_BINARY, line_count, joint, add_ID, carriage_return, report_ID);
                switch (manager)
                {
                    case PHRASES:
                        if (case_sensitive)
                        {
                            exit_code = a.arranging_to_produce_text();
                        }
                        else
                        {
                            exit_code = a.case_insensitive_arranging_to_produce_text();
                        }

                        break;
                    case INTEGERS:
                        exit_code = a.arranging_integers_to_produce_text();
                        break;
                    case FLOATING_POINT_NUMBERS:
                        exit_code = a.arranging_doubles_to_produce_text();
                        break;
                    default:
                        break;
                }

            }

            w.Stop();
            if (ticktock)
            {
                Console.Write( w.ElapsedTicks );
                Console.WriteLine(" ticks");
            }

            if (remove_temporary_files)
            {
                File.Delete(TEMPORARY_BINARY);
                File.Delete(INTERMEDIATE_FILE);

                if (brackets)
                {
                    File.Delete(INTERMEDIATE_FILE_2);
                }

                if (joint_type > 0)
                {
                    File.Delete(INTERMEDIATE_FILE_3);
                }
                else
                {
                    if (insert_both_bytes_and_nibbles)
                    {
                        File.Delete(INTERMEDIATE_FILE_3);
                    }

                }

            }

            System.Environment.Exit(exit_code);
        }

    }
}
