﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Narfi
{
    class DividingExceptDakuten : Dividing
    {
        const int ASCII_SIZE = 1;
        const int BELL = 0x07;
        const int BELL_ASCII_KATAKANA_COUNT = 224;
        const int KANA_DAKUTEN = 0xDE;
        const int KANA_HANDAKUTEN = 0xDF;
        const int KANA_KUTEN = 0xA1;
        const int KATAKANA_SIZE = 1;
        const int ZENKAKU_SIZE = 2;

        int[] BELL_ASCII_KATAKANA = {
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 
            0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 
            0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 
            0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 
            0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 
            0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 
            0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 
            0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 
            0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF
        };
/*
H is a substitute for dakuten. I is a substitute for handankuten. Please see sortk.txt
ﾞH
ﾟI
*/
        public DividingExceptDakuten(FieldsList fields_list, char s, byte c, bool e, char substitute_for_dakuten, char substitute_for_handakuten) : base(fields_list, s, c, e)
        {
            int m = (int)substitute_for_dakuten;
            int n = (int)substitute_for_handakuten;

            BELL_ASCII_KATAKANA[m] = BELL;
            BELL_ASCII_KATAKANA[n] = BELL;
        }

        public DividingExceptDakuten(FieldsList fields_list, int c, char s, bool e, char substitute_for_dakuten, char substitute_for_handakuten) : base(fields_list, c, s, e)
        {
            int m = (int)substitute_for_dakuten;
            int n = (int)substitute_for_handakuten;

            BELL_ASCII_KATAKANA[m] = BELL;
            BELL_ASCII_KATAKANA[n] = BELL;
        }

        public override int editing(string p, int field_number)
        {
            smashed_to_pieces(p);

            vector_to_array_editing(remove_control, field_number);
            return field_count;
        }

        string remove_control(string field)
        {
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            int k = 0;
            int n;

            try
            {
                Encoding MS932 = Encoding.GetEncoding(932);
                byte[] buffer = MS932.GetBytes(field);
                int size = MS932.GetByteCount(field);
                byte[] combo = new byte[size];

                for (int i = 0; i < size; ++i)
                {
                    c = buffer[i];

                    if (kanji)
                    {
                        combo[k] = firstByte;
                        combo[ k + 1 ] = c;
                        k += ZENKAKU_SIZE;
                        kanji = false;
                        continue;
                    }

                    if (0x81 <= c && c <= 0xFC)
                    {
                        if (KANA_KUTEN <= c && c < KANA_DAKUTEN)
                        {
                            combo[k] = c;
                            k += KATAKANA_SIZE;
                            kanji = false;
                        }
                        else if (c == KANA_DAKUTEN)
                        {
                            continue; // skipping a combining character (dakuten)
                        }
                        else if (c == KANA_HANDAKUTEN)
                        {
                            continue; // skipping a combining character (handakuten)
                        }
                        else
                        {
                            firstByte = c;
                            kanji = true;
                        }

                    }
                    else
                    {
                        n = BELL_ASCII_KATAKANA[c];
                        if (n == BELL)
                        {
                            continue; // skipping control characters and substitutes for combining characters
                        }
                        else
                        {
                            combo[k] = c;
                            k += ASCII_SIZE;
                        }

                        kanji = false;
                    }

                } // for (int i = 0; i < size; ++i)

                field = MS932.GetString(combo);
                k = field.IndexOf('\0');
                if (k >= 0)
                {
                    field = field.Substring(0, k);
                }

            }
            catch (ArgumentException e572)
            {
                Error.handler( e572.ToString() );
            }
            catch (Exception e573)
            {
                Error.handler( e573.ToString() );
            }

            return field;
        }

    }
}
