﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Narfi
{
    class StandardInput
    {
        const bool LISTED = true;
        const bool NO_APPENDING = false;
        const int ERROR_CODE = -1;
        const string CLEAR = "0";

        bool full_house, remove_separators, way_of_arranging;
        char ascii, separator;
        Encoding encoded;
        int field_count;
        string pattern;

        public StandardInput(Encoding e, string p, int c, char a, char s, bool r, bool w)
        {
            encoded = e;
            pattern = p;
            field_count = c;
            separator = s;
            remove_separators = r;
            way_of_arranging = w;

            int u = 0x00FF & a;
            ascii = (char)u;

            int k = pattern.IndexOf(CLEAR);
            full_house = (k >= 0) ? false : true;
        }

        public int standard_input_counting()
        {
            byte few_fields;
            Dividing d;
            FieldsList f;
            int exit_code = 0;
            int t, shared_field_count;
            string line;

            try
            {
                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    line = Console.ReadLine();
                    if (line == null)
                    {
                        return ERROR_CODE;
                    }

                    f = new FieldsList();
                    d = new Dividing(f, separator, few_fields, remove_separators);
                    shared_field_count = d.mapping_fields_list(line);

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        d = new Dividing(f, separator, few_fields, remove_separators);
                        t = d.mapping_fields_list(line);
                        if (t != shared_field_count)
                        {
                            Console.WriteLine( d.unite(line, LISTED) );
                        }

                    }

                }
                else
                {
                    line = Console.ReadLine();
                    if (line == null)
                    {
                        return ERROR_CODE;
                    }

                    f = new FieldsList();
                    d = new Dividing(f, field_count, separator, remove_separators);
                    shared_field_count = d.mapping_fields_list(line);

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        d = new Dividing(f, field_count, separator, remove_separators);
                        t = d.mapping_fields_list(line);
                        if (t != shared_field_count)
                        {
                            Console.WriteLine( d.unite(line, LISTED) );
                        }

                    }

                }

            }
            catch (IOException e149)
            {
                Error.handler( e149.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e150)
            {
                Error.handler( e150.ToString() );
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int standard_input_abolishing(int field_number)
        {
            byte few_fields;
            Dividing d;
            FieldsList f;
            int exit_code = 0;
            string line;

            try
            {
                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new DividingEditing(f, separator, few_fields, remove_separators, ascii);
                        d.editing(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new DividingEditing(f, field_count, separator, remove_separators, ascii);
                        d.editing(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }

            }
            catch (IOException e165)
            {
                Error.handler( e165.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e167)
            {
                Error.handler( e167.ToString() );
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int standard_input_output(int field_number)
        {
            byte few_fields;
            Dividing d;
            FieldsList f;
            int exit_code = 0;
            string line;

            try
            {
                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new DividingRemoving(f, separator, few_fields, remove_separators);
                        d.editing(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new DividingRemoving(f, field_count, separator, remove_separators);
                        d.editing(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }

            }
            catch (IOException e274)
            {
                Error.handler( e274.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e275)
            {
                Error.handler( e275.ToString() );
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int standard_input_output_except_dakuten(int field_number, bool substituted)
        {
            byte few_fields;
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            Dividing d;
            FieldsList f;
            int exit_code = 0;
            string line;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new DividingExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                        d.editing(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new DividingExceptDakuten(f, field_count, separator, remove_separators, dakuten, handakuten);
                        d.editing(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }

            }
            catch (IOException e566)
            {
                Error.handler( e566.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e567)
            {
                Error.handler( e567.ToString() );
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int standard_input_extracting_decimal(int field_number)
        {
            byte few_fields;
            Dividing d;
            FieldsList f;
            int exit_code = 0;
            string line;

            try
            {
                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new Dividing(f, separator, few_fields, remove_separators);
                        d.extracting_decimal_from_field(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new Dividing(f, field_count, separator, remove_separators);
                        d.extracting_decimal_from_field(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }

            }
            catch (IOException e154)
            {
                Error.handler( e154.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e155)
            {
                Error.handler( e155.ToString() );
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int standard_input_extracting_integer(int field_number)
        {
            byte few_fields;
            Dividing d;
            FieldsList f;
            int exit_code = 0;
            string line;

            try
            {
                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new Dividing(f, separator, few_fields, remove_separators);
                        d.extracting_integer_from_field(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = Console.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        f = new FieldsList();
                        d = new Dividing(f, field_count, separator, remove_separators);
                        d.extracting_integer_from_field(line, field_number);
                        Console.WriteLine( d.unite(line, LISTED) );
                    }

                }

            }
            catch (IOException e156)
            {
                Error.handler( e156.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e157)
            {
                Error.handler( e157.ToString() );
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int standard_input_output()
        {
            byte few_fields;
            Dividing d;
            FieldsList f;
            int exit_code = 0;
            string line;

            try
            {
                if (way_of_arranging)
                {
//        full_house = true;

                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            f = new FieldsList();
                            d = new Dividing(f, separator, few_fields, remove_separators);
                            Console.WriteLine( d.reverse_unite(line) );
                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            f = new FieldsList();
                            d = new Dividing(f, field_count, separator, remove_separators);
                            Console.WriteLine( d.reverse_unite(line) );
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                f = new FieldsList();
                                d = new Dividing(f, separator, few_fields, remove_separators);
                                Console.WriteLine( d.unite(line) );
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = Console.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                f = new FieldsList();
                                d = new Dividing(f, pattern, separator, few_fields, remove_separators);
                                Console.WriteLine( d.unite(line) );
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = Console.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            f = new FieldsList();
                            d = new Dividing(f, pattern, field_count, separator, remove_separators);
                            Console.WriteLine( d.unite(line) );
                        }

                    }

                }

            }
            catch (IOException e151)
            {
                Error.handler( e151.ToString() );
                exit_code = ERROR_CODE;
            }
            catch (Exception e152)
            {
                Error.handler( e152.ToString() );
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

    }
}
