﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Rufeol
{
    class RemoveFromLine
    {
        const int BLOCK_SIZE = 2560;
        const int ERROR_CODE = 1;
        const int FULL_LEVEL = 1;
        const int LF_SIZE = 1;
        const int FILE_ERROR_CODE = -2;
        const long FILE_ERROR = -1L;
        const string CRLF = "\x0D\x0A";
        const string EMPTY = "";
        const string LF = "\x0A";
        const string TEMPORARY_BINARY = "binary.tmp";

        byte BELL = 0x07;
        byte ring_peek;
        byte[] buffer;
        byte[] combo;
        int bufferSize, fullLevel, ring_count, unnecessary;
        long file_size;
        Queue<byte> ring = new Queue<byte>();
        string fileName;

        private bool ringBeginsWithBell()
        {
            bool b = false;
            ring_peek = ring.Peek();

            if (ring_peek == BELL)
            {
                b = true;
            }

            return b;
        }

        public int renameBinaryFile(string name, bool no_removing)
        {
            int exit_code = 0;
            int size;

            buffer = new byte[BLOCK_SIZE];

            try
            {
                File.Delete(name);
                Stream temporary = File.OpenRead(TEMPORARY_BINARY);
                Stream recent = File.OpenWrite(name);

                while ( ( size = temporary.Read(buffer, 0, BLOCK_SIZE) ) > 0 )
                {
                    recent.Write(buffer, 0, size);
                }

                recent.Close();
                temporary.Close();

                if (! no_removing)
                {
                    File.Delete(TEMPORARY_BINARY);
                }

            }
            catch (IOException e112)
            {
                Error.handler( e112.ToString() );
                exit_code = FILE_ERROR_CODE;
            }
            catch (Exception e38)
            {
                Error.handler( e38.ToString() );
                exit_code = FILE_ERROR_CODE;
            }

            return exit_code;
        }

        private void prepareRing()
        {
            ring.Clear();

            for (int i = 0; i < ring_count; ++i)
            {
                ring.Enqueue(BELL);
            }

        }

        private long fileSize()
        {
            long size = FILE_ERROR;

            try
            {
                FileStream stream = new FileStream(fileName, FileMode.Open, FileAccess.Read);

                if (stream.CanSeek)
                {
                    size = stream.Length;
                }
                else
                {
                    Error.handler("stream.CanSeek error: false");
                }

                stream.Close();
            }
            catch (IOException e39)
            {
                Error.handler( e39.ToString() );
            }
            catch (Exception e40)
            {
                Error.handler( e40.ToString() );
            }

            return size;
        }

        public RemoveFromLine(string f, int b, int l, int u)
        {
            bufferSize = b;
            fileName = f;
            fullLevel = l;
            unnecessary = u;

            file_size = fileSize();
            if (file_size == FILE_ERROR)
            {
                Error.handler("file_size == FILE_ERROR");
            }
            else
            {
                int size = (int)file_size;
                if(bufferSize > size) {
                    bufferSize = size;
                }

                buffer = new byte[bufferSize];
                combo = new byte[bufferSize];
            }

        }

        public bool insertCRLF()
        {
            bool full = false;
            byte c;
            int count = 0;
            int k = 0;

            try
            {
                File.Delete(TEMPORARY_BINARY);
                Stream temporaryFile = File.OpenWrite(TEMPORARY_BINARY);
                Stream binaryFile = File.OpenRead(fileName);

                // ring buffer
                ring_count = unnecessary + LF_SIZE;
                prepareRing();

                binaryFile.Read(buffer, 0, bufferSize);

                for(int m = 0; m < bufferSize; ++m) {
                    c = buffer[m];
                    if (c == 0x0A)
                    {
                        count += 1;
                        if (count >= fullLevel)
                        {
                            full = true;
                        }

                    }

                    ring.Dequeue();
                    ring.Enqueue(c);

                    if (full)
                    {
                        combo[k++] = 0x0D; // CR
                        combo[k++] = 0x0A; // LF
                        prepareRing();
                        count = 0;
                        full = false;
                    }
                    else
                    {
                        if ( ringBeginsWithBell() ) // ring.Peek()
                        {
                            continue; // skipping 07h
                        }
                        else
                        {
                            combo[k++] = ring_peek;
                        }

                    }

                } // for(int m = 0; m < bufferSize; ++m)

                temporaryFile.Write(combo, 0, k);
                temporaryFile.Close();
                binaryFile.Close();
            }
            catch (IOException e41)
            {
                Console.Error.Write("OpenRead() error: ");
                Console.Error.WriteLine(fileName);
                Error.handler( e41.ToString() );
                return false;
            }
            catch (Exception e42)
            {
                Error.handler( e42.ToString() );
                return false;
            }

            return true;
        }

        public bool insertLF()
        {
            bool full = false;
            byte c;
            int count = 0;
            int k = 0;

            try
            {
                Stream temporaryFile = File.OpenWrite(TEMPORARY_BINARY);
                Stream binaryFile = File.OpenRead(fileName);

                // ring buffer
                ring_count = unnecessary + LF_SIZE;
                prepareRing();

                binaryFile.Read(buffer, 0, bufferSize);

                for(int m = 0; m < bufferSize; ++m) {
                    c = buffer[m];
                    if (c == 0x0A)
                    {
                        count += 1;
                        if (count >= fullLevel)
                        {
                            full = true;
                        }

                    }

                    ring.Dequeue();
                    ring.Enqueue(c);

                    if (full)
                    {
                        combo[k++] = 0x0A; // LF
                        prepareRing();
                        count = 0;
                        full = false;
                    }
                    else
                    {
                        if ( ringBeginsWithBell() ) // ring.Peek()
                        {
                            continue; // skipping 07h
                        }
                        else
                        {
                            combo[k++] = ring_peek;
                        }

                    }

                } // for(int m = 0; m < bufferSize; ++m)

                temporaryFile.Write(combo, 0, k);
                temporaryFile.Close();
                binaryFile.Close();
            }
            catch (IOException e43)
            {
                Console.Error.Write("OpenRead() error: ");
                Console.Error.WriteLine(fileName);
                Error.handler( e43.ToString() );
                return false;
            }
            catch (Exception e44)
            {
                Error.handler( e44.ToString() );
                return false;
            }

            return true;
        }

    }
}
