﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

/*
SjisEuc C# Version 1.04
Converting the text from Shift_JIS to EUC-JP
Copyright 2019 Sogaya

Visual C# 2008 Express
Visual Studio 2010

SjisEuc assumes that the little-endian architecture underlies operating systems (Linux, Windows).
*/

namespace SjisEuc
{
    class Program
    {
        static void Main(string[] args)
        {
            const bool REMOVE_ANCIENT_FILE = false;
            const bool WITH_COMMENTS = true;
            const bool WITHOUT_COMMENTS = false;
            const double VERSION = 1.04;
            const string ACQUIRE_TABLE_NAME_FROM_TEXT = "-a";
            const string CRLF = "-CRLF";
            const string DIFFERENT_TIME = "-t-";
            const string DO_NOT_REMOVE_0DH = "-L-";
            const string DO_NOT_REMOVE_TEMPORARY_BINARY_FILE = "-y";
            const string DO_NOT_SEARCH_TABLE = "-d";
            const string DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE = "-w-";
            const string EMPTY = "";
            const string IDENTICAL_TIME = "-t";
            const string LF = "-LF";
            const string LINE_FEED = "-L";
            const string NO_SKIPPING = "-s-";
            const string SEARCH_TABLE = "-d-";
            const string SETTING_TEXT = "sete.txt";
            const string SJIS_EUC_BIN = "MS932~EUC-JP.bin";
            const string SJIS_EUC_TEXT = "SjisEuc.txt";
            const string SKIPPING = "-s";
            const string TEMPORARY_BINARY = "binary.tmp";
            const string TEMPORARY_BINARY_FILE = "-b";
            const string VERSION_WITHOUT_COMMENTS = "-V";
            const string VERSION_WITH_COMMENTS = "-VC";
            const string WHICH_TABLE = "-w";

            bool carriage_return = false;
            bool directory_has_priority = false;
            bool identical_time = false;
            bool no_removing = false;
            bool no_searching = false;
            bool produce_binary_file = false;
            bool skip_control_characters = false;
            bool file_to_be_read = true;
            bool which_table = false;
            Guide g = new Guide();
            int exit_code = 0;
            int file_name_size;
            string binary_name = SJIS_EUC_BIN;
            string new_name = SJIS_EUC_TEXT;
            string document_name = EMPTY;

// Acquiring parameters from SETTING_TEXT unless the user removes it.
            AcquireFromText a = new AcquireFromText(SETTING_TEXT, binary_name, document_name, new_name, carriage_return, identical_time, no_removing, no_searching, produce_binary_file, skip_control_characters, which_table);
            a.acquiring();
            binary_name = a.binaryName();
            new_name = a.newName();
            document_name = a.textName();
            carriage_return = a.carriageReturn();
            identical_time = a.identicalTime();
            no_removing = a.noRemoving();
            no_searching = a.noSearching();
            produce_binary_file = a.produceBinary();
            skip_control_characters = a.skipping();
            which_table = a.whichTable();

            foreach (string argument in args)
            {
                switch (argument)
                {
                    case ACQUIRE_TABLE_NAME_FROM_TEXT:
                        directory_has_priority = true;
                        break;
                    case CRLF:
                        carriage_return = true;
                        break;
                    case DIFFERENT_TIME:
                        identical_time = false;
                        break;
                    case DO_NOT_REMOVE_0DH:
                        carriage_return = true;
                        break;
                    case DO_NOT_REMOVE_TEMPORARY_BINARY_FILE:
                        no_removing = true;
                        break;
                    case DO_NOT_SEARCH_TABLE:
                        no_searching = true;
                        break;
                    case DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE:
                        which_table = false;
                        break;
                    case IDENTICAL_TIME:
                        identical_time = true;
                        break;
                    case LF:
                        carriage_return = false;
                        break;
                    case LINE_FEED:
                        carriage_return = false;
                        break;
                    case NO_SKIPPING:
                        skip_control_characters = false;
                        break;
                    case SEARCH_TABLE:
                        no_searching = false;
                        break;
                    case SKIPPING:
                        skip_control_characters = true;
                        break;
                    case TEMPORARY_BINARY_FILE:
                        produce_binary_file = true;
                        break;
                    case VERSION_WITH_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITH_COMMENTS);
                        System.Environment.Exit(exit_code);
                        break;
                    case VERSION_WITHOUT_COMMENTS:
                        exit_code = g.versionGuide(VERSION, WITHOUT_COMMENTS);
                        System.Environment.Exit(exit_code);
                        break;
                    case WHICH_TABLE:
                        which_table = true;
                        break;
                    default:
                        if (file_to_be_read)
                        {
                            document_name = argument;
                            file_to_be_read = false;
                        }
                        else
                        {
                            new_name = argument;
                        }

                        break;
                } // switch

            } // foreach

            SearchTable s = new SearchTable(binary_name, no_searching, directory_has_priority, which_table);

            file_name_size = document_name.Length;
            if (file_name_size == 0)
            {
                StandardInput i = new StandardInput(s);

                if (skip_control_characters)
                {
                    exit_code = i.standard_input_output_skipping();
                }
                else
                {
                    exit_code = i.standard_input_output();
                }

            }
            else
            {
                Regulate r = new Regulate(s, document_name);

                if (identical_time)
                {
                    r.produceBinaryFile(document_name, REMOVE_ANCIENT_FILE);
                    r.renameBinaryFile(document_name, no_removing);
                }

                if (carriage_return)
                {
                    if (produce_binary_file)
                    {
                        if (skip_control_characters)
                        {
                            exit_code = r.skippingControlCharactersToProduceEUC_CRLF(TEMPORARY_BINARY);
                        }
                        else
                        {
                            exit_code = r.readingToProduceEUC_CRLF(TEMPORARY_BINARY);
                        }

                        r.renameBinaryFile(new_name, no_removing);
                    }
                    else
                    {
                        if (skip_control_characters)
                        {
                            exit_code = r.skippingControlCharactersToProduceEUC_CRLF(new_name);
                        }
                        else
                        {
                            exit_code = r.readingToProduceEUC_CRLF(new_name);
                        }

                    }

                }
                else
                {
                    if (produce_binary_file)
                    {
                        if (skip_control_characters)
                        {
                            exit_code = r.skippingControlCharactersToProduceEUC(TEMPORARY_BINARY);
                        }
                        else
                        {
                            exit_code = r.readingToProduceEUC(TEMPORARY_BINARY);
                        }

                        r.renameBinaryFile(new_name, no_removing);
                    }
                    else
                    {
                        if (skip_control_characters)
                        {
                            exit_code = r.skippingControlCharactersToProduceEUC(new_name);
                        }
                        else
                        {
                            exit_code = r.readingToProduceEUC(new_name);
                        }

                    }

                }

            }

            System.Environment.Exit(exit_code);
        }

    }
}
