﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SjisEuc
{
    class StandardInput
    {
        const int ASCII_SIZE = 1;
        const int BELL = 0x07;
        const int DOUBLE_KANJI = 4; // SJIS_KANJI_SIZE + EUC_KANJI_SIZE;
        const int ERROR_CODE = 1;
        const int KANA_HANDAKUTEN = 0xDF;
        const int KANA_KUTEN = 0xA1;
        const int KANJI_SIZE = 2;
        const int KATAKANA_SIZE = 1;
        const string SJIS_EUC_TEXT = "SjisEuc.txt";
        const string TEMPORARY_BINARY = "binary.tmp";

        int[] BELL_ASCII_KATAKANA = {
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 
            0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 
            0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 
            0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 
            0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 
            0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 
            0x07, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 
            0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 
            0xC0, 0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 
            0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC, 0xDD, 0xDE, 0xDF
        };

        SearchTable s;

        private int inverted(byte[] c)
        {
            int k = (int)c[0];
            k = k << 8;
            k += c[1];
            return k;
        }

        public int standard_input_output()
        {
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] w = {0, 0};
            Encoding MS932 = Encoding.GetEncoding(932);
            int size;
            int exit_code = 0;
            string line;

            SparseEUC euc = new SparseEUC();
            if (! euc.setting(s))
            {
                Console.Error.Write("standard_input_output() error: ");
                Console.Error.WriteLine("euc.setting(s)");
                return ERROR_CODE;
            }

            try
            {
                Stream standard_output = Console.OpenStandardOutput();

                for (; ; )
                {
                    line = Console.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = MS932.GetBytes(line);
                    size = MS932.GetByteCount(line);
                    for (int i = 0; i < size; ++i)
                    {
                        c = buffer[i];

                        if (kanji)
                        {
                            a[0] = firstByte;
                            a[1] = buffer[i];
                            w[0] = euc.working( inverted(a), 0);
                            w[1] = euc.working( inverted(a), 1);
                            standard_output.Write(w, 0, KANJI_SIZE);
                            kanji = false;
                            continue;
                        }

                        if (0x81 <= c && c<= 0xFC)
                        {
                            if (KANA_KUTEN <= c && c <= KANA_HANDAKUTEN)
                            {
                                w[0] = euc.working(c, 0);
                                w[1] = 0;
                                standard_output.Write(w, 0, KATAKANA_SIZE);
                                kanji = false;
                            }
                            else
                            {
                                firstByte = c;
                                kanji = true;
                            }

                        }
                        else
                        {
                            w[0] = euc.working(c, 0);
                            w[1] = 0;
                            standard_output.Write(w, 0, ASCII_SIZE);
                            kanji = false;
                        }

                    } // for (int i = 0; i < size; ++i)

                    w[0] = 0x0D;
                    w[1] = 0x0A;
                    standard_output.Write(w, 0, 2);
                } //for (; ; )

            }
            catch (IOException e107)
            {
                Error.handler(e107.ToString());
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public int standard_input_output_skipping()
        {
            bool kanji = false;
            byte c;
            byte firstByte = 0x40;
            byte[] a = {0, 0};
            byte[] buffer;
            byte[] w = {0, 0};
            Encoding MS932 = Encoding.GetEncoding(932);
            int size;
            int exit_code = 0;
            string line;

            SparseEUC euc = new SparseEUC();
            if (! euc.setting(s))
            {
                Console.Error.Write("standard_input_output_skipping() error: ");
                Console.Error.WriteLine("euc.setting(s)");
                return ERROR_CODE;
            }

            try
            {
                Stream standard_output = Console.OpenStandardOutput();

                for (; ; )
                {
                    line = Console.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = MS932.GetBytes(line);
                    size = MS932.GetByteCount(line);
                    for (int i = 0; i < size; ++i)
                    {
                        c = buffer[i];

                        if (kanji)
                        {
                            a[0] = firstByte;
                            a[1] = buffer[i];
                            w[0] = euc.working( inverted(a), 0);
                            w[1] = euc.working( inverted(a), 1);
                            standard_output.Write(w, 0, KANJI_SIZE);
                            kanji = false;
                            continue;
                        }

                        if (0x81 <= c && c<= 0xFC)
                        {
                            if (KANA_KUTEN <= c && c <= KANA_HANDAKUTEN)
                            {
                                w[0] = euc.working(c, 0);
                                w[1] = 0;
                                standard_output.Write(w, 0, KATAKANA_SIZE);
                                kanji = false;
                            }
                            else
                            {
                                firstByte = c;
                                kanji = true;
                            }

                        }
                        else
                        {
                            if (BELL_ASCII_KATAKANA[c] == BELL)
                            {
                                continue; // skipping control characters
                            }
                            else
                            {
                                w[0] = euc.working(c, 0);
                                w[1] = 0;
                                standard_output.Write(w, 0, ASCII_SIZE);
                            }

                            kanji = false;
                        }

                    } // for (int i = 0; i < size; ++i)

                    w[0] = 0x0D;
                    w[1] = 0x0A;
                    standard_output.Write(w, 0, 2);
                } //for (; ; )

            }
            catch (IOException e108)
            {
                Error.handler(e108.ToString());
                exit_code = ERROR_CODE;
            }

            return exit_code;
        }

        public StandardInput(SearchTable search_table)
        {
            s = search_table;
        }

    }
}
