﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Sortk
{
    class Line
    {
        const int BUFFER_SIZE_ERROR = -1;
        const int LINE_COUNT_ERROR = -1;
        const int NOT_FOUND = -1;
        const int SEPARATOR_SIZE = 1;
        const long FILE_ERROR = -1L;
        const string EMPTY = "";

        bool carriageReturn, measure;
        byte[] buffer;
        Encoding encoded;
        int bufferSize, field_count, lineCount, maximum, newLineLength, point;
        int lineNumber = 1;
        int maximumLineLength = 0;
        int[] fieldLengthCount = {0, 0};
        string fileName;

        public bool counting(int separator)
        {
            bool b = false;
            int shared_field_count;
            string line = EMPTY;

            try
            {
                StreamReader file = new StreamReader(fileName, encoded);

                line = file.ReadLine();
                if (line == null)
                {
                    return false;
                }

                buffer = encoded.GetBytes(line);
                fieldLengthCount = maximumSpan(separator, line.Length);
                shared_field_count = fieldLengthCount[1];

                for (; ; )
                {
                    line = file.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    buffer = encoded.GetBytes(line);
                    fieldLengthCount = maximumSpan(separator, line.Length);
                    field_count = fieldLengthCount[1];
                    if (field_count != shared_field_count)
                    {
                        Console.WriteLine(line);
                    }

                }

                file.Close();
            }
            catch (IOException e277)
            {
                Console.Error.WriteLine(fileName);
                Error.handler( e277.ToString() );
            }
            catch (Exception e278)
            {
                Error.handler( e278.ToString() );
            }

            return b;
        }

        public bool reading()
        {
            bool b = false;
            int n = 0;
            string line = EMPTY;

            try
            {
                StreamReader file = new StreamReader(fileName, encoded);

                for (; ; )
                {
                    line = file.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    n += 1;
                    if (n == point)
                    {
                        b = true;
                        break;
                    }

                }

                file.Close();

                if (measure)
                {
                    Console.Write(fileName);
                    Console.Write(": ");
                    Console.Write(lineCount);
                    if (n == 1)
                    {
                        Console.WriteLine(" line");
                    }
                    else
                    {
                        Console.WriteLine(" lines");
                    }

                    Console.Write("maximum LF-to-LF span of ");
                    Console.Write(maximumLineLength);
                    Console.Write(" bytes (line-");
                    Console.Write(lineNumber);
                    Console.WriteLine(")");
                }

                Console.WriteLine(line);
            }
            catch (IOException e279)
            {
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(fileName);
                Error.handler( e279.ToString() );
            }
            catch (Exception e280)
            {
                Error.handler( e280.ToString() );
            }

            return b;
        }

        public bool reading(int separator)
        {
            bool b = false;
            int fieldLength;
            int n = 0;
            string line = EMPTY;

            try
            {
                StreamReader file = new StreamReader(fileName, encoded);

                for (; ; )
                {
                    line = file.ReadLine();
                    if (line == null)
                    {
                        break;
                    }

                    n += 1;
                    if (n == point)
                    {
                        b = true;
                        break;
                    }

                }

                file.Close();

                if (measure)
                {
                    Console.Write(fileName);
                    Console.Write(": ");
                    Console.Write(lineCount);
                    if (n == 1)
                    {
                        Console.WriteLine(" line");
                    }
                    else
                    {
                        Console.WriteLine(" lines");
                    }

                    buffer = encoded.GetBytes(line);
                    fieldLengthCount = maximumSpan(separator, line.Length);
                    fieldLength = fieldLengthCount[0] - SEPARATOR_SIZE;
                    Console.Write("maximum separator-to-separator span of ");
                    Console.Write(fieldLength);
                    Console.Write(" bytes (line-");
                    Console.Write(point);
                    Console.WriteLine(")");
                }

                Console.WriteLine(line);
            }
            catch (IOException e281)
            {
                Console.Error.WriteLine(fileName);
                Error.handler( e281.ToString() );
            }
            catch (Exception e282)
            {
                Error.handler( e282.ToString() );
            }

            return b;
        }

        public int checkBufferSize(bool stop)
        {
            int buffer_size = BUFFER_SIZE_ERROR;
            string s;

            if (bufferSize >= 0)
            {
                buffer_size = bufferSize;
            }
            else
            {
                if (stop)
                {
                    s = string.Format("checkBufferSize() error: {0:D}", bufferSize);
                    throw new Exception(s);
                }

            }

            return buffer_size;
        }

        public int checkLineCount(bool stop)
        {
            int line_count = LINE_COUNT_ERROR;
            string s;

            if (lineCount >= 0)
            {
                line_count = lineCount;
            }
            else
            {
                if (stop)
                {
                    s = string.Format("checkLineCount() error: {0:D}", lineCount);
                    throw new Exception(s);
                }

            }

            return line_count;
        }

        public int fieldCount()
        {
            return fieldLengthCount[1];
        }

        public int maximumSpan()
        {
            int c, m;
            int count = 0;
            int k;
            int max = 0;
            int trace = NOT_FOUND;

            carriageReturn = false;
            newLineLength = 1;
            for (k = 0; k < bufferSize && trace == NOT_FOUND; ++k)
            {
                c = (int)buffer[k];

                switch(c)
                {
                    case 0x0A:
                        count += 1;

                        if (carriageReturn)
                        {
                            max = k - 1 + newLineLength;
                        }
                        else
                        {
                            max = k + newLineLength;
                        }

                        lineNumber = count;
                        trace = k; // trace != NOT_FOUND
                        break;
                    case 0x0D:
                        carriageReturn = true;
                        newLineLength = 2;
                        break;
                    default:
                        break;
                }

            }

            for ( ; k < bufferSize; ++k)
            {
                c = (int)buffer[k];

                switch(c)
                {
                    case 0x0A:
                        count += 1;
                        m = k - trace;
                        if (m > max)
                        {
                            max = m;
                            lineNumber = count;
                        }

                        trace = k;
                        break;
                    default:
                        break;
                }

            }

            lineCount = count;
            return max;
        }

        public int[] maximumSpan(int separator, int size)
        {
            int c, m;
            int count = 0;
            int k;
            int max = 0;
            int trace = NOT_FOUND;
            int[] lengthCount = {0, 0};

            for (k = 0; k < size; ++k)
            {
                c = (int)buffer[k];

                if(c == separator)
                {
                    count += 1;
                    max = k + SEPARATOR_SIZE;
                    trace = k;
                    k = trace + 1;
                    break;
                }

            }

            for( ; k < size; ++k)
            {
                c = (int)buffer[k];

                if(c == separator)
                {
                    count += 1;
                    m = k - trace;
                    if(m > max)
                    {
                        max = m;
                    }

                    trace = k;
                }

            }

            lengthCount[0] = max;
            lengthCount[1] = count;
            return lengthCount;
        }

        public int maximumLineSize()
        {
            return maximumLineLength;
        }

        public Line(Encoding e, string f)
        {
            encoded = e;
            fileName = f;
            measure = false;
            point = 1;
            bufferSize = (int)fileSize();
            if (bufferSize >= 0)
            {
                buffer = new byte[bufferSize];

                Stream binaryFile = File.OpenRead(fileName);
                binaryFile.Read(buffer, 0, bufferSize);
                maximum = maximumSpan(); // carriageReturn, lineCount, lineNumber
                newLineLength = carriageReturn ? 2 : 1;
                maximumLineLength = maximum - newLineLength;
                binaryFile.Close();
            }

        }

        public Line(Encoding e, string f, int p, bool m)
        {
            encoded = e;
            fileName = f;
            measure = m;
            point = p;
            bufferSize = (int)fileSize();
            if (bufferSize >= 0)
            {
                buffer = new byte[bufferSize];

                Stream binaryFile = File.OpenRead(fileName);
                binaryFile.Read(buffer, 0, bufferSize);
                maximum = maximumSpan(); // carriageReturn, lineCount, lineNumber
                newLineLength = carriageReturn ? 2 : 1;
                maximumLineLength = maximum - newLineLength;
                binaryFile.Close();
            }

        }

        public long fileSize()
        {
            long size = FILE_ERROR;

            try
            {
                FileStream f = new FileStream(fileName, FileMode.Open, FileAccess.Read);
                size = (long) f.Length;
                f.Close();
            }
            catch (IOException e283)
            {
                Console.Error.Write("FileStream() error: ");
                Console.Error.WriteLine(fileName);
                Error.handler( e283.ToString() );
            }
            catch (Exception e284)
            {
                Error.handler( e284.ToString() );
            }

            return size;
        }

    }
}
