﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;

namespace Sortk
{
    class SummaryDetails
    {
        const bool LISTED = true;
        const bool NO_APPENDING = false;
        const char CLEAR = '0';
        const char JOINT = ' '; // Sortk uses ' ' as a joint.
        const char GAP = ' ';
        const int ERROR_CODE = -1;
        const string EMPTY = "";
        const string SUTAI_XML = "sutai.xml";
        const string XML_VERSION_ENCODING = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>";
        const string XML_STYLESHEET_TYPE = "<?xml-stylesheet type=\"text/xsl\" href=\"sutai.xsl\" ?>";

        bool full_house, remove_separators;
        char joint, quote, separator;
        Encoding encoded;
        Encoding UTF_8 = Encoding.UTF8;
        int field_count, upper_limit;
        string file_name, pattern;
        XmlDocument x;
        XmlElement record, sutai;

        private string postediting(string s)
        {
            char c;
            StringBuilder os = new StringBuilder();
            for (int i = 0; i < s.Length; ++i)
            {
                c = s[i];
                switch (c)
                {
                    case ' ':
                        os.Append('!'); // It is necessary to replace ' ' with an exclamation mark so that Sortk can use ' ' as a joint.
                        break;
                    case '!':
                        os.Append(JOINT); // Sortk uses ' ' as a joint. Please note that if the text includes an exclamation mark, Sortk confuses it with a joint.
                        break;
                    default:
                        if (c == joint)
                        {
                            os.Append(JOINT); // Sortk uses ' ' as a joint.
                        }
                        else
                        {
                            os.Append(c);
                        }

                        break;
                }

            }

            return os.ToString();
        }

        public SummaryDetails(Encoding e, string p, string f, char j, char qu, char s, int c, int u, bool v)
        {
            encoded = e;
            pattern = p;
            file_name = f;
            joint = j;
            quote = qu;
            separator = s;
            field_count = c;
            upper_limit = u;
            remove_separators = v;

            int q = pattern.IndexOf(CLEAR);
            full_house = (q >= 0) ? false : true;

            x = new XmlDocument();
            sutai = x.CreateElement("sutai_xml");
            x.AppendChild(sutai);
        }

        public int counting()
        {
            byte few_fields;
            int f, shared_field_count;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    line = file.ReadLine();
                    if (line == null)
                    {
                        return ERROR_CODE;
                    }

                    line_count += 1;

                    d = new SegmentedSummary(separator, few_fields, remove_separators);
                    d.setting(joint, quote);
                    d.segmented_summary(line);
                    shared_field_count = d.fieldCount();

                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(separator, few_fields, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        f = d.fieldCount();
                        if (f != shared_field_count)
                        {
                            Console.WriteLine(line);
                        }

                    }

                }
                else
                {
                    line = file.ReadLine();
                    if (line == null)
                    {
                        return ERROR_CODE;
                    }

                    line_count += 1;

                    d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                    d.setting(joint, quote);
                    d.segmented_summary(line);
                    shared_field_count = d.fieldCount();

                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        f = d.fieldCount();
                        if (f != shared_field_count)
                        {
                            Console.WriteLine(line);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e319)
            {
                Error.handler( e319.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e320)
            {
                Error.handler( e320.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int line_for_each_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            if (d.uniqueCombo(hashSet))
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            if ( d.uniqueCombo(hashSet) )
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        if ( d.uniqueCombo(hashSet) )
                        {
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e313)
            {
                Error.handler( e313.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e314)
            {
                Error.handler( e314.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int line_for_each_double_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.doubleElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.doubleElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_double_from_field(line);
                        if ( d.uniqueFieldsList(hashSet) )
                        {
                            record = d.doubleElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e315)
            {
                Error.handler( e315.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e316)
            {
                Error.handler( e316.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int line_for_each_integer_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.integerElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            if ( d.uniqueFieldsList(hashSet) )
                            {
                                record = d.integerElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_integer_from_field(line);
                        if ( d.uniqueFieldsList(hashSet) )
                        {
                            record = d.integerElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e317)
            {
                Error.handler( e317.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e318)
            {
                Error.handler( e318.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e287)
            {
                Error.handler( e287.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e288)
            {
                Error.handler( e288.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_doubles()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_double_from_field(line);
                        record = d.doubleElement(x, line);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e289)
            {
                Error.handler( e289.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e290)
            {
                Error.handler( e290.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_integers()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_integer_from_field(line);
                        record = d.integerElement(x, line);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e291)
            {
                Error.handler( e291.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e292)
            {
                Error.handler( e292.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_adding_ID()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line, line_count);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e303)
            {
                Error.handler( e303.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e304)
            {
                Error.handler( e304.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_doubles_adding_ID()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_double_from_field(line);
                            record = d.doubleElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_double_from_field(line);
                        record = d.doubleElement(x, line, line_count);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e305)
            {
                Error.handler( e305.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e306)
            {
                Error.handler( e306.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int selecting_integers_adding_ID()
        {
            byte few_fields;
            FieldsList f;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new SegmentedSummary(f, pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.extracting_integer_from_field(line);
                            record = d.integerElement(x, line, line_count);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        f = new FieldsList();
                        d = new SegmentedSummary(f, pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.extracting_integer_from_field(line);
                        record = d.integerElement(x, line, line_count);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e307)
            {
                Error.handler( e307.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e308)
            {
                Error.handler( e308.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int skipping()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line, GAP);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e405)
            {
                Error.handler( e405.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e406)
            {
                Error.handler( e406.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int skipping_adding_ID()
        {
            byte few_fields;
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            record = d.phraseElement(x, line, line_count, GAP);
                            sutai.AppendChild(record);
                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        record = d.phraseElement(x, line, line_count, GAP);
                        sutai.AppendChild(record);
                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e407)
            {
                Error.handler( e407.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e408)
            {
                Error.handler( e408.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int skipping_showing_line_for_each_summary()
        {
            byte few_fields;
            HashSet<string> hashSet = new HashSet<string>();
            int line_count = 0;
            SegmentedSummary d;
            string line;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);
                StreamWriter newFile = new StreamWriter(SUTAI_XML, NO_APPENDING, UTF_8);
                newFile.Write(XML_VERSION_ENCODING);
                newFile.Write(XML_STYLESHEET_TYPE);

                if (field_count <= 0x00FF)
                { // 0 to 255
                    few_fields = (byte)field_count;

                    if (full_house)
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            d.compactCombo(GAP);
                            if (d.uniqueCombo(hashSet))
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            d = new SegmentedSummary(pattern, separator, few_fields, remove_separators);
                            d.setting(joint, quote);
                            d.segmented_summary(line);
                            d.compactCombo(GAP);
                            if ( d.uniqueCombo(hashSet) )
                            {
                                record = d.phraseElement(x, line);
                                sutai.AppendChild(record);
                            }

                        }

                        newFile.Write( x.OuterXml );
                    }

                }
                else
                {
                    for (; ; )
                    {
                        line = file.ReadLine();
                        if (line == null)
                        {
                            break;
                        }

                        line_count += 1;
                        if(line_count > upper_limit) {
                            break;
                        }

                        d = new SegmentedSummary(pattern, field_count, separator, remove_separators);
                        d.setting(joint, quote);
                        d.segmented_summary(line);
                        d.compactCombo(GAP);
                        if ( d.uniqueCombo(hashSet) )
                        {
                            record = d.phraseElement(x, line);
                            sutai.AppendChild(record);
                        }

                    }

                    newFile.Write( x.OuterXml );
                }

                file.Close();
                newFile.Close();
            }
            catch (IOException e409)
            {
                Error.handler( e409.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                line_count = ERROR_CODE;
            }
            catch (Exception e410)
            {
                Error.handler( e410.ToString() );
                line_count = ERROR_CODE;
            }

            return line_count;
        }

        public int shift_jis_selecting(string new_name, bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e419)
            {
                Error.handler( e419.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e420)
            {
                Error.handler( e420.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e417)
                {
                    Error.handler( e417.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e418)
                {
                    Error.handler( e418.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_selecting_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e429)
            {
                Error.handler( e429.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e430)
            {
                Error.handler( e430.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e431)
                {
                    Error.handler( e431.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e432)
                {
                    Error.handler( e432.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_selecting_for_case_insensitive_search(string new_name, bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e421)
            {
                Error.handler( e421.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e422)
            {
                Error.handler( e422.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e423)
                {
                    Error.handler( e423.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e424)
                {
                    Error.handler( e424.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_selecting_for_case_insensitive_search_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e433)
            {
                Error.handler( e433.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e434)
            {
                Error.handler( e434.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e435)
                {
                    Error.handler( e435.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e436)
                {
                    Error.handler( e436.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping(string new_name, bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e411)
            {
                Error.handler( e411.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e412)
            {
                Error.handler( e412.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e415)
                {
                    Error.handler( e415.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e416)
                {
                    Error.handler( e416.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e437)
            {
                Error.handler( e437.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e438)
            {
                Error.handler( e438.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e439)
                {
                    Error.handler( e439.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e440)
                {
                    Error.handler( e440.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping_for_case_insensitive_search(string new_name, bool add_ID, bool descending, bool report_ID, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            RemoveControl d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new RemoveControl(f, pattern, separator, few_fields, remove_separators);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new RemoveControl(f, pattern, field_count, separator, remove_separators);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e425)
            {
                Error.handler( e425.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e426)
            {
                Error.handler( e426.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e427)
                {
                    Error.handler( e427.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e428)
                {
                    Error.handler( e428.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int shift_jis_skipping_for_case_insensitive_search_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            int new_name_size;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakuten d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakuten(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakuten(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e441)
            {
                Error.handler( e441.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e442)
            {
                Error.handler( e442.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            new_name_size = new_name.Length;
            if (new_name_size == 0)
            {
                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        Console.WriteLine(items[i]);
                    }

                }
                else
                {
                    items.ForEach(g => Console.WriteLine(g));
                }

            }
            else
            {
                try
                {
                    StreamWriter newFile = new StreamWriter(new_name, NO_APPENDING, encoded);

                    if (descending)
                    {
                        for (int i = items.Count - 1; i >= 0; --i)
                        {
                            newFile.WriteLine(items[i]);
                        }

                    }
                    else
                    {
                        items.ForEach(g => newFile.WriteLine(g));
                    }

                    newFile.Close();
                }
                catch (IOException e481)
                {
                    Error.handler( e481.ToString() );
                    Console.Error.Write("StreamWriter() error: ");
                    Console.Error.WriteLine(new_name);
                    exit_code = ERROR_CODE;
                }
                catch (Exception e482)
                {
                    Error.handler( e482.ToString() );
                    exit_code = ERROR_CODE;
                }

            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_selecting_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e534)
            {
                Error.handler( e534.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e535)
            {
                Error.handler( e535.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            try
            {
                ComboAppend ca = new ComboAppend('\n');

                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        ca.separatorFollows(items[i]);
                    }

                }
                else
                {
                    foreach (string s  in items)
                    {
                        ca.separatorFollows(s);
                    }

                }

                File.Delete(new_name);
                Stream newFile = File.OpenWrite(new_name);
                byte[] combo = encoded.GetBytes( ca.built() );
                newFile.Write(combo, 0, combo.Length);
                newFile.Close();
            }
            catch (IOException e536)
            {
                Error.handler( e536.ToString() );
                Console.Error.Write("StreamWriter() error: ");
                Console.Error.WriteLine(new_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e537)
            {
                Error.handler( e537.ToString() );
                exit_code = ERROR_CODE;
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_selecting_for_case_insensitive_search_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite() );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite() );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e538)
            {
                Error.handler( e538.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e539)
            {
                Error.handler( e539.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            try
            {
                ComboAppend ca = new ComboAppend('\n');

                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        ca.separatorFollows(items[i]);
                    }

                }
                else
                {
                    foreach (string s  in items)
                    {
                        ca.separatorFollows(s);
                    }

                }

                File.Delete(new_name);
                Stream newFile = File.OpenWrite(new_name);
                byte[] combo = encoded.GetBytes( ca.built() );
                newFile.Write(combo, 0, combo.Length);
                newFile.Close();
            }
            catch (IOException e540)
            {
                Error.handler( e540.ToString() );
                Console.Error.Write("StreamWriter() error: ");
                Console.Error.WriteLine(new_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e541)
            {
                Error.handler( e541.ToString() );
                exit_code = ERROR_CODE;
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_skipping_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            Block b;
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<Block> blockArray = new List<Block>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new Block(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new Block(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e542)
            {
                Error.handler( e542.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e543)
            {
                Error.handler( e543.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            try
            {
                ComboAppend ca = new ComboAppend('\n');

                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        ca.separatorFollows(items[i]);
                    }

                }
                else
                {
                    foreach (string s  in items)
                    {
                        ca.separatorFollows(s);
                    }

                }

                File.Delete(new_name);
                Stream newFile = File.OpenWrite(new_name);
                byte[] combo = encoded.GetBytes( ca.built() );
                newFile.Write(combo, 0, combo.Length);
                newFile.Close();
            }
            catch (IOException e544)
            {
                Error.handler( e544.ToString() );
                Console.Error.Write("StreamWriter() error: ");
                Console.Error.WriteLine(new_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e545)
            {
                Error.handler( e545.ToString() );
                exit_code = ERROR_CODE;
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

        public int euc_jp_skipping_for_case_insensitive_search_except_dakuten(string new_name, bool add_ID, bool descending, bool report_ID, bool substituted, bool unique_summary)
        {
            byte few_fields;
            CheckID c = new CheckID();
            char dakuten = (char)0xDE;
            char handakuten = (char)0xDF;
            FieldsList f;
            int exit_code = 0;
            int line_count = 0;
            List<UpperCaseBlock> blockArray = new List<UpperCaseBlock>();
            List<string> items = new List<string>();
            ExceptDakutenEUC d;
            string line, point, summary;
            string previous = EMPTY;
            UpperCaseBlock b;

            if (substituted)
            {
                dakuten = 'H';
                handakuten = 'I';
            }

            try
            {
                StreamReader file = new StreamReader(file_name, encoded);

                if (add_ID)
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(c, line, summary, line_count.ToString() );
                            blockArray.Add(b);
                        }

                    }

                }
                else
                {
                    if (field_count <= 0x00FF)
                    { // 0 to 255
                        few_fields = (byte)field_count;

                        if (full_house)
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }
                        else
                        {
                            for (; ; )
                            {
                                line = file.ReadLine();
                                if (line == null)
                                {
                                    break;
                                }

                                line_count += 1;
                                if(line_count > upper_limit) {
                                    break;
                                }

                                f = new FieldsList();
                                d = new ExceptDakutenEUC(f, pattern, separator, few_fields, remove_separators, dakuten, handakuten);
                                d.setting(joint, quote);
                                d.editing(line);
                                summary = postediting( d.unite(GAP) );
                                b = new UpperCaseBlock(line, summary);
                                blockArray.Add(b);
                            }

                        }

                    }
                    else
                    {
                        for (; ; )
                        {
                            line = file.ReadLine();
                            if (line == null)
                            {
                                break;
                            }

                            line_count += 1;
                            if(line_count > upper_limit) {
                                break;
                            }

                            f = new FieldsList();
                            d = new ExceptDakutenEUC(f, pattern, field_count, separator, remove_separators, dakuten, handakuten);
                            d.setting(joint, quote);
                            d.editing(line);
                            summary = postediting( d.unite(GAP) );
                            b = new UpperCaseBlock(line, summary);
                            blockArray.Add(b);
                        }

                    }

                }

                file.Close();
            }
            catch (IOException e546)
            {
                Error.handler( e546.ToString() );
                Console.Error.Write("StreamReader() error: ");
                Console.Error.WriteLine(file_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e547)
            {
                Error.handler( e547.ToString() );
                exit_code = ERROR_CODE;
            }

            blockArray.Sort();

            if (unique_summary)
            {
                for (int i = 0; i < blockArray.Count; ++i)
                {
                    point = blockArray[i].ToString();
                    if (point == previous)
                    {
                        continue;
                    }
                    else
                    {
                        items.Add(point);
                    }

                    previous = point;
                }

            }
            else
            {
                blockArray.ForEach( g => items.Add( g.ToString() ) );
            }

            try
            {
                ComboAppend ca = new ComboAppend('\n');

                if (descending)
                {
                    for (int i = items.Count - 1; i >= 0; --i)
                    {
                        ca.separatorFollows(items[i]);
                    }

                }
                else
                {
                    foreach (string s  in items)
                    {
                        ca.separatorFollows(s);
                    }

                }

                File.Delete(new_name);
                Stream newFile = File.OpenWrite(new_name);
                byte[] combo = encoded.GetBytes( ca.built() );
                newFile.Write(combo, 0, combo.Length);
                newFile.Close();
            }
            catch (IOException e548)
            {
                Error.handler( e548.ToString() );
                Console.Error.Write("StreamWriter() error: ");
                Console.Error.WriteLine(new_name);
                exit_code = ERROR_CODE;
            }
            catch (Exception e549)
            {
                Error.handler( e549.ToString() );
                exit_code = ERROR_CODE;
            }

            if (add_ID)
            {
                if (report_ID)
                {
                    c.reportID();
                }

            }

            return exit_code;
        }

    }
}
